;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |          DMX routines for RGB frame          |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Type      : ASM                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________DMX system init
;Initializes DMX system
;Modified registers:
;	system
;Call method
;	CALL Dmx_Init
Dmx_Init:
	BANKSEL SP1BRGH			;Bank 2
	MOVLF c_DMX_BAUDH,SP1BRGH	;Copy DMX baud rate MSB
	MOVLF c_DMX_BAUDL,SP1BRGL	;Copy DMX baud rate LSB
	MOVLF c_DMX_TXSTA,TX1STA	;Set USART for DMX mode
	MOVLF c_DMX_RXSTA,RC1STA	;
	MOVLF c_DMX_BCON,BAUD1CON	;Set Baud generator for DMX
        BANKSEL PORTA			;Bank 0
	BSF f_DMXM_BRK			;Set first step of DMX master/slave sub
	MOVLF c_DMX_ADDRH,w_DMX_ADDR	;Load default DMX slave address
	MOVLF c_DMX_ADDRL,w_DMX_ADDR+1	;
	RETURN

;_______________________________________________________DMX start transmitter
;Starts DMX transmission in master mode.
;Modified registers:
;	system
;Call method
;	CALL Dmx_StartT
Dmx_StartT:
        BANKSEL b_DMX_CTRL		;Bank 0
	BTFSS f_DMX_ON			;Check if DMX is enabled
	RETURN				; No, exit
	BTFSS f_DMX_MASTER		; Yes, check if DMX is in master mode
	RETURN				;  No, exit
	BANKSEL PIR3			;Bank 14
	BCF PIR3,RC1IF			;Clear serial receive IRQ
	BCF PIE3,RC1IE			;Disable serial receive IRQ
	BANKSEL PORTC			;Bank 0
	BSF c_TX_LATC,c_TX_PIN		;Set TX pin
	BCF c_TX_TRIS,c_TX_PIN		;Set TX pin as output
	BANKSEL c_TX_PPS		;Bank 61
	CLRF c_TX_PPS			;TX pin is digital I/O
	BANKSEL PORTA			;Bank 0
	RETURN
;_______________________________________________________DMX start receiver
;Starts DMX reception if in slave mode.
;Modified registers:
;	system
;Call method
;	CALL Dmx_StartR
Dmx_StartR:
        BANKSEL b_DMX_CTRL		;Bank 0
	BTFSS f_DMX_ON			;Check if DMX is enabled
	RETURN				; No, exit
	BTFSC f_DMX_MASTER		; Yes, check if DMX is in slave mode
	RETURN				;  No, exit
	BCF c_DIR_PORT,c_DIR_PIN	;Yes, clear DIR pin (Receive)
	BCF c_TX_LATC,c_TX_PIN		;Clear TX pin
	BSF c_TX_TRIS,c_TX_PIN		;Set TX pin as input
	BANKSEL c_TX_PPS		;Bank 61
	CLRF c_TX_PPS			;TX pin is digital I/O
	BANKSEL PIR3			;Bank 14
	BCF PIR3,RC1IF			;Clear serial receive IRQ
	BSF PIE3,RC1IE			;Enable serial receive IRQ
	BANKSEL PORTA			;Bank 0
	RETURN

;_______________________________________________________DMX address conversion from binary to decimal
;Reads the pushbutton, fast ans slow press
;Modified registers
;	system
;	b_DMX_HNDT
;	b_DMX_TENT
;	b_DMX_UNIT
;Call method
;	Write address to w_DMX_ADDR
;	CALL Dmx_Addr_BCD
Dmx_Addr_BCD:
	BANKSEL w_DMX_ADDR		;Bank 0
	CLRF b_DMX_HNDT			;Clear decimal address before updating it
	CLRF b_DMX_TENT			;
	CLRF b_DMX_UNIT			;
	MOVFF w_DMX_ADDR,R0H		;Copy binary address to temp
	MOVFF w_DMX_ADDR+1,R0L		;
DAB_HUN	MOVLW .100			;							    10
	SUBWF R0L,w			;AddressL -100 >= 0?
	BC DAB_HU1			; Yes, Lsb greater than 100, no need to check Msb
	MOVWF R0L			; No, Lsb <100, save Lsb
	MOVLW .1			;
	SUBWF R0H,w			;AddressH -1 >= 0?
	BNC DAB_DEC			; No
	MOVWF R0H			; Yes, save Msb
	INCF b_DMX_HNDT,f		;Hundreds++
	GOTO DAB_HUN			;Repeat
DAB_HU1	MOVWF R0L			;Save Lsb
	INCF b_DMX_HNDT,f		;Hundreds++
	GOTO DAB_HUN			;Repeat
DAB_DEC	MOVLW .100			;Restore correct Lsb value				    44
	ADDWF R0L,f			;
DAB_DC2	MOVLW .10			;
	SUBWF R0L,w			;AddressL-10>0? (AddressH is already out of question)
	BNC DAB_UNI			; No
	MOVWF R0L			; Yes, save
	INCF b_DMX_TENT,f		; Tents++
	GOTO DAB_DC2			; Repeat
DAB_UNI	MOVF R0L,w			;Address =0?						    76
	BZ DAB_END			; Yes
DAB_UN2	INCF b_DMX_UNIT,f		; No, units++
	DECFSZ R0L,f			; Address-1=0?
	GOTO DAB_UN2			;  No,repeat
DAB_END	RETURN				;  Yes, exit						    35

;_______________________________________________________DMX address conversion from decimal to binary
;Reads the pushbutton, fast ans slow press
;Modified registers
;	system
;	w_DMX_ADDR
;Call method
;	Write address to b_DMX_HNDT, b_DMX_TENT, b_DMX_UNIT
;	CALL Dmx_Addr_DBC
Dmx_Addr_DBC:
    	BANKSEL w_DMX_ADDR		;Bank 0
	CLRF w_DMX_ADDR			;Clear binary address before updating it
	CLRF w_DMX_ADDR+1		;
	CLRC				;Clear carry
	RLF b_DMX_TENT,w		;Tens * 2				(18)
	MOVWF w_DMX_ADDR+1		;Address= Tens * 2
	SWAPF b_DMX_HNDT,w		;Hundreds * 16				(144)
	ADDWF b_DMX_HNDT,w              ;(Hundreds * 16) + Hundreds		(153)
	ADDWF w_DMX_ADDR+1,f		;Address+= (Hundreds * 17)		(171)
	RLF w_DMX_ADDR+1,f		;Address * 2				(342)
	RLF w_DMX_ADDR,f		;-> carry
	SWAPF b_DMX_HNDT,w		;Hundreds * 16				(144)
	ADDWF b_DMX_TENT,w		;+Tens					(153)
	ADDWF w_DMX_ADDR+1,f		;Address+= (Hundreds * 16 + Tens)	(495)
	SKPNC				;
	INCF w_DMX_ADDR,f		;-> carry
	RLF w_DMX_ADDR+1,f		;Address * 2				(990)
	RLF w_DMX_ADDR,f		;-> carry
	MOVF b_DMX_UNIT,w		;					(9)
	ADDWF w_DMX_ADDR+1,f		;Address+= Units			(999)
	SKPNC				;
	INCF w_DMX_ADDR,f		;-> carry
	RETURN

;_______________________________________________________DMX master send
;Sends the DMX frame
;NOTE: use LSBit as Start bit because DMX512 mandates it to be always 0 (AN1076 is either
;	wrong or I did not understand it, but based on figure on page 7, I will assume the first),
;	put MSBit in TX0STA,TX9D (RLF + carry check), then wait at least 8us (4us is the bit time)
;	before next transmit. With polarity high for idle, this will give 2 "1" end bits. This time
;	equals to 64 instructions, including IRQ management. That's minimum 36 instructions just
;	before getting back here, so let's check how much more we need. Note that the "mark between
;	packets" can be anything from nothing to 1 full second, so there isn't a very strict timing
;	here. I want to keep it as short as reasonably possible anyway.
;Modified registers
;	system
;	b_DMX_CTRL
;Call method
;	CALL Dmx_Master
Dmx_Master:
	BANKSEL b_DMX_CTRL		;Bank 0
	BTFSS f_DMX_ON			;Check if DMX enabled
	RETURN				; No, exit
	BTFSS f_DMX_MASTER		; Yes, check if DMX master mode
	RETURN				;  No, exit
	BCF f_IRQ_TIMER0_ON		;  Yes, clear Timer0 IRQ flag
	BTFSC f_DMXM_BRK		;  Check if Break send mode
	GOTO DXM_BR			;   Yes
	BTFSC f_DMXM_MAB		;   No, check if Mark-After-Break send mode
	GOTO DXM_MB			;    Yes
	BTFSC f_DMXM_STR		;    No, check if Start packet send mode
	GOTO DXM_TMT			;     Yes
DXM_EN	ANDLF c_DMXM_CLR,b_DMX_CTRL	;        No, end of sequence, clear all modes
	BSF f_DMXM_BRK			;Reset mode to first
	BSF c_TX_LATC,c_TX_PIN		;Set TX pin
	BCF c_TX_TRIS,c_TX_PIN		;Set TX pin as output
	BANKSEL c_TX_PPS		;Bank 61
	CLRF c_TX_PPS			;TX pin is digital I/O
	BANKSEL PORTA			;Bank 0
	RETURN
DXM_BR	BSF c_DIR_PORT,c_DIR_PIN	;Set DIR pin (Transmission)
	BSF c_TX_LATC,c_TX_PIN		;Set TX pin
	BCF c_TX_TRIS,c_TX_PIN		;Set TX pin as output
	BANKSEL c_TX_PPS		;Bank 61
	CLRF c_TX_PPS			;TX pin is digital I/O
	BANKSEL PORTA			;Bank 0
	BCF f_DMXM_BRK			;Clear Break mode
	BSF f_DMXM_MAB			;Set Mark-After-Break mode
	BCF c_TX_LATC,c_TX_PIN		;Clear TX pin (Start Break)
	BANKSEL TMR0L			;Bank 11
	MOVLF c_TMR0_BRKH,TMR0H		;
	MOVLF c_TMR0_BRKL,TMR0L		;Load Timer0 with Break timing
	BSF T0CON0,T0EN			;Start Timer0
	BANKSEL PIR0			;Bank 14
	BCF PIR0,TMR0IF			;Clear Timer0 IRQ flag
	BSF PIE0,TMR0IE			;Enable Timer0 IRQ
	BANKSEL PWM3DCH			;Bank 6
	MOVFF PWM3DCH,R1H		;load RED duty value from timer
	MOVFF PWM4DCH,R1L		;load GREEN duty value from timer
	MOVFF PWM5DCH,R2H		;load BLUE duty value from timer
	BANKSEL PORTA			;Bank 0
	RETURN
DXM_MB	BSF c_TX_LATC,c_TX_PIN		;Set TX pin (Start MAB)
	BANKSEL TMR0L			;Bank 11
	MOVLF c_TMR0_MABH,TMR0H		;Reset Timer0 duration
	MOVLF c_TMR0_MABL,TMR0L		;
	BSF T0CON0,T0EN			;Start Timer0
	BANKSEL b_DMX_CTRL		;Bank 0
	BCF f_DMXM_MAB			;Clear Mark-After-Break mode
	BSF f_DMXM_STR			;Set Start mode
	BANKSEL PIR0			;Bank 14
	BCF PIR0,TMR0IF			;Clear Timer0 IRQ flag
	BSF PIE0,TMR0IE			;Enable Timer0 IRQ
	BANKSEL PORTA			;Bank 0
	RETURN
DXM_TMT
	BANKSEL RC4PPS			;Bank 62
	MOVLF c_RC4PPS,RC4PPS		;Route TX1 to pin RC4
	BANKSEL TX1STA			;Bank 2
	BSF TX1STA,TXEN			;Enable EUSART TX
	MOVLF c_DMX_START,TX1REG	;Load start byte into TX buffer
	NOP				;Delay to allow data to be moved from TXREG to TSR
	NOP				;
DXM_ST	BTFSS TX1STA,TRMT		;Wait for the data to be transmitted
	GOTO DXM_ST			;
	MOVF R1H,w			;Copy actual RED value into WREG
	MOVWF TX1REG			;Load RED value into TX buffer
	NOP				;Delay to allow data to be moved from TXREG to TSR
	NOP				;
DXM_RD	BTFSS TX1STA,TRMT		;Wait for the data to be transmitted
	GOTO DXM_RD			;
	MOVF R1L,w			;Copy actual GREEN value into WREG
	MOVWF TX1REG			;Load GREEN byte into TX buffer
	NOP				;Delay to allow data to be moved from TXREG to TSR
	NOP				;
DXM_GN	BTFSS TX1STA,TRMT		;Wait for the data to be transmitted
	GOTO DXM_GN			;
	MOVF R2H,w			;Copy actual BLUE value into WREG
	MOVWF TX1REG			;Load BLUE byte into TX buffer
	NOP				;Delay to allow data to be moved from TXREG to TSR
	NOP				;
DXM_BL	BTFSS TX1STA,TRMT		;Wait for the data to be transmitted
	GOTO DXM_BL			;
	BANKSEL PORTA			;Bank 0
	GOTO DXM_EN			;Finish

;_______________________________________________________DMX slave receive
;Receives the DMX frame
;Should be called only in IRQ
;Modified registers
;	system
;	b_DMX_CTRL
;	b_DMX_ERR
;	b_DMX_RED
;	b_DMX_GRN
;	b_DMX_BLU
;	w_DMX_CNT
;Call method
;	CALL Dmx_Slave
Dmx_Slave:
	BANKSEL b_DMX_CTRL		;Bank 0
	BTFSS f_DMX_INH			;Check if DMX inhibit enabled (this happens only if slave)
	RETURN				; No, exit
	BANKSEL RC1STA			;Bank 2
	BTFSS RC1STA,OERR		;Check if Overrun error present
	GOTO DXS_CK			; No
DXS_RS	BCF RC1STA,CREN			; Yes, reset error
	MOVFW RC1REG			; Read the bad data to clear the RX buffer
	MOVFW RC1REG			;
	MOVFW RC1REG			;
	BSF RC1STA,CREN			; Reenable reception
	BANKSEL b_DMX_CTRL		; Bank 0
	ANDLF c_DMXS_CLR,b_DMX_CTRL	; Reset sequence and wait next Break
	BSF f_DMXM_BRK			; Set framing error wait
	RETURN
DXS_CK	
	BANKSEL b_DMX_CTRL		;Bank 0
	MOVF b_DMX_ERR,w		;Check if errors reached limit (error count is reset after
	SUBLW c_DMX_ERR			;      complete reception of DMX frame)
	BNZ DXS_CK1			; No, keep going
	CLRF b_DMX_ERR			; Yes, reset error count
	CLRF b_DMX_RED			; Clear DMX data
	CLRF b_DMX_GRN			; Clear DMX data
	CLRF b_DMX_BLU			; Clear DMX data
	CLRF w_DMX_CNT			; Reset address counter
	CLRF w_DMX_CNT+1		;
	BANKSEL RC1STA			; Bank 2
	GOTO DXS_RS			; Reset sequence
DXS_CK1	BTFSC f_DMXM_BRK		;Check if waiting for framing error (Break code)
	GOTO DXS_FE			; Yes
	BTFSC f_DMXM_STR		; No, check if waiting for start packet
	GOTO DXS_ST			;  Yes
	BTFSC f_DMXS_RD			;  No, check if waiting for RED packet
	GOTO DXS_RD			;   Yes
	BTFSC f_DMXS_GN			;   No, check if waiting for GREEN packet
	GOTO DXS_GN			;    Yes
	BTFSC f_DMXM_MAB		;    No, check if waiting for BLUE packet
	GOTO DXS_BL			;     Yes
	BSF f_DMXM_BRK			;     No, first start, set framing error wait
DXS_FE					;Framing error receive
	BANKSEL RC1STA			;Bank 2
	BTFSS RC1STA,FERR		;Check if framing error present
	GOTO DXS_FE1			; No, skip the rest
	BANKSEL w_DMX_CNT		; Bank 0
	CLRF w_DMX_CNT			; Reset address counter
	CLRF w_DMX_CNT+1		;
	ANDLF c_DMXS_CLR,b_DMX_CTRL	;Clear sequence
	BSF f_DMXM_STR			;Set start packet wait
	INCF b_DMX_ERR,f		;Increment error count
DXS_FE1
	BANKSEL RC1STA			;Bank 2
	MOVFW RC1REG			;Read the bad data to clear the RX buffer
	MOVFW RC1REG			;
	MOVFW RC1REG			;
	BANKSEL PORTA			;Bank 0
	RETURN
DXS_ST					;Start packet receive
	BANKSEL RC1STA			;Bank 2
	BTFSC RC1STA,FERR		;Check if framing error present
	GOTO DXS_FE			; Yes, reset sequence
	MOVFW RC1REG			; No, load the data
	BANKSEL b_DMX_CTRL		; Bank 0
	SUBLW c_DMX_START		; Check if start packet is equal to defined one. If not, refuse it
	BNZ DXS_ST1			; NOTE: may need to cope with RDM packets in a different way,
					;       but for now let's do it this way. It's easier
	BCF f_DMXM_STR			;Clear start packet wait
	BSF f_DMXS_RD			;Set RED packet wait
	RETURN
DXS_ST1 BSF f_DMXM_BRK			;Set framing error wait (reset sequence and wait next Break)
	BCF f_DMXM_STR			;Clear start packet wait
	CLRF w_DMX_CNT			;Reset address counter
	CLRF w_DMX_CNT+1		;
	GOTO DXS_FE1			;Read the bad data to clear the RX buffer
DXS_RD	INCFSZ w_DMX_CNT+1,f		;RED packet receive, increment address counter
	GOTO DXS_RD1			;
	INCF w_DMX_CNT,f		;
DXS_RD1	MOVFW w_DMX_CNT			;Compare address counter MSB with DMX address MSB
        SUBWF w_DMX_ADDR,w		;
        BNZ DXS_FE1			;MSBs are different, no need to go further
        MOVFW w_DMX_CNT+1		;Compare LSBs
        SUBWF w_DMX_ADDR+1,w		;
	BNZ DXS_FE1			;LSBs are different, these are not the bits you are looking for
	BANKSEL RC1STA			;Bank 2
	BTFSC RC1STA,FERR		;Check if framing error present
	GOTO DXS_FE			; Yes, reset sequence
	MOVFW RC1REG			; No, read the data
	BANKSEL w_DMX_CNT		; Bank 0
	MOVWF b_DMX_RED			; Save the data into RED PWM setpoint
	BCF f_DMXS_RD			; Clear RED packet wait
	BSF f_DMXS_GN			; Set GREEN packet wait
	RETURN
DXS_GN					;GREEN packet receive
	BANKSEL RC1STA			;Bank 2
	BTFSC RC1STA,FERR		;Check if framing error present
	GOTO DXS_FE			; Yes, reset sequence
	MOVFW RC1REG			; No, read the data
	BANKSEL w_DMX_CNT		; Bank 0
	MOVWF b_DMX_GRN			; Save the data into GREEN PWM setpoint
	BCF f_DMXS_GN			; Clear GREEN packet wait
	BSF f_DMXM_MAB			; Set BLUE packet wait
	RETURN
DXS_BL					;BLUE packet receive
	BANKSEL RC1STA			;Bank 2
	BTFSC RC1STA,FERR		;Check if framing error present
	GOTO DXS_FE			; Yes, reset sequence
	MOVFW RC1REG			; No, read the data
	BANKSEL w_DMX_CNT		; Bank 0
	MOVWF b_DMX_BLU			; Save the data into BLUE PWM setpoint
	CLRF b_DMX_ERR			; Reset DMX error counter
	ANDLF c_DMXS_CLR,b_DMX_CTRL	; Clear sequence
	BSF f_DMXM_BRK			; Set framing error wait
	GOTO DXS_FE1			; Read the data to clear the RX buffer

;_______________________________________________________DMX receive check
;Checks for errors outside the IRQ calls. If overrun error present, resets the reception.
;Modified registers
;	system
;	b_DMX_CTRL
;	b_DMX_ERR
;	b_DMX_RED
;	b_DMX_GRN
;	b_DMX_BLU
;	w_DMX_CNT
;Call method
;	CALL Dmx_Slave
Dmx_Rxck:
	BANKSEL b_DMX_CTRL		;Bank 0
	BTFSS f_DMX_INH			;Check if DMX inhibit enabled (this happens only if slave)
	RETURN				; No, exit
	BANKSEL RC1STA			;Bank 2
	BTFSC RC1STA,OERR		;Check if Overrun error present
	GOTO DXR_RS			; Yes
DXR_CK					; No
	BANKSEL b_DMX_CTRL		;Bank 0
	MOVF b_DMX_ERR,w		;Check if errors reached limit (error count is reset after
	SUBLW c_DMX_ERR			;      complete reception of DMX frame)
	BNZ DXR_END			; No, keep going
	CLRF b_DMX_RED			; Yes, clear DMX data
	CLRF b_DMX_GRN			;
	CLRF b_DMX_BLU			;
	BANKSEL RC1STA			;Bank 2
DXR_RS	BCF RC1STA,CREN			;Overrun present, reset error
	MOVFW RC1REG			;Read the bad data to clear the RX buffer
	MOVFW RC1REG			;
	MOVFW RC1REG			;
	BSF RC1STA,CREN			;Reenable reception
	BANKSEL b_DMX_CTRL		;Bank 0
	ANDLF c_DMXS_CLR,b_DMX_CTRL	;Reset sequence and wait next Break
	BSF f_DMXM_BRK			;Set framing error wait
	CLRF b_DMX_ERR			;Reset error count
	CLRF w_DMX_CNT			;Reset address counter
	CLRF w_DMX_CNT+1		;
DXR_END	RETURN
