;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |          I/O routines for RGB frame          |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Type      : ASM                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________Ports init
;Clear PORTS and initialize all I/O related registers
;Modified registers:
;	system
;Call method
;	CALL Io_Init
Io_Init:
        BANKSEL PORTA			;Bank 0
	CLRF PORTA			;Clear PORTA
	CLRF PORTC			;Clear PORTC
	CLRF LATA			;Clear PORTA Data Latch
	CLRF LATC			;Clear PORTC Data Latch
	MOVLF c_TRISA,TRISA		;Set PORTA I/O
	MOVLF c_TRISC,TRISC		;Set PORTC I/O
	BANKSEL ANSELA			;Bank 62
	MOVLF c_ANSA,ANSELA		;Set PORTA D/A mode
	MOVLF c_ANSC,ANSELC		;Set PORTC D/A mode
	MOVLF c_WPUA,WPUA		;Set PORTA Pull-Up mode
	MOVLF c_WPUC,WPUC		;Set PORTC Pull-Up mode
	MOVLF c_ODCA,ODCONA		;Set PORTA Open-Drain mode
	MOVLF c_ODCC,ODCONC		;Set PORTC Open-Drain mode
	MOVLF c_SLRA,SLRCONA		;Set PORTA Slew Rate mode
	MOVLF c_SLRC,SLRCONC		;Set PORTC Slew Rate mode
	MOVLF c_INLVA,INLVLA		;Set PORTA Input level mode
	MOVLF c_INLVC,INLVLC		;Set PORTC Input level mode
	MOVLF c_RC0PPS,RC0PPS		;Route PWM5 to pin RC0
	MOVLF c_RC1PPS,RC1PPS		;Route PWM4 to pin RC1
	MOVLF c_RC2PPS,RC2PPS		;Route PWM3 to pin RC2
	CLRF RC4PPS			;RC4 is digital I/O
	BANKSEL RX1DTPPS		;Bank 61
	MOVLF c_RX1PPS,RX1DTPPS		;Route pin RC5 to RX1
	MOVLF c_TX1PPS,TX1CKPPS		;Route pin RC4 to TX1
	BANKSEL PORTA			;Bank 0
	BSF f_SW_TIM2			;set push timeout (start with push not pressed and static)
	RETURN

;_______________________________________________________Pushbutton management
;Reads the pushbutton, fast ans slow press
;Modified registers
;	system
;	b_SW_FAST
;	b_SW_SLOW
;	b_SW_STAT
;Call method
;	CALL Push_Read
Push_Read:
	BANKSEL c_PUSH_PORT		;Bank 0
	BTFSC c_PUSH_PORT,c_PUSH_PIN	;Check if pushbutton pressed
	GOTO SW_ST1			; Yes
	BTFSC f_SW_TIM2			; No, check push timeout
	RETURN				;  Set, nothing to do
	BTFSS f_SW_STAT			;  Clear, check push status
	GOTO SW_CL1			;   Clear
	DECFSZ b_SW_FAST,f		;   Set, decrement buffer
	GOTO SW_CL1			;    Not zero
	BCF f_SW_STAT			;    Zero, clear push status
	BSF f_SW_CHG			;    Set status change flag
	BCF f_MENU_BLINK		;    Disable blink counter
SW_CL1	BTFSC f_SW_TIM1			;Check push timer toggle
	GOTO SW_CL2			; Set
	BSF f_SW_TIM1			; Clear, set it
	RETURN				; Exit
SW_CL2	INCFSZ b_SW_TMR,f		;Increase timeout counter
	RETURN				; Not zero, exit
	BSF f_SW_TOUT			; Zero, set push timeout flag
	BSF f_SW_TIM2			; Set push timeout timer flag
	BTFSS f_DMX_ON			; Check if DMX is ON
	RETURN				;  No, end
	BTFSC f_DMX_MASTER		;  Yes, check if master
	RETURN				;   Yes, end
	BSF f_DMX_INH			;   No, set inhibit (DMX SLAVE, REACTIVATE INHIBIT after timeout)
	CALL Dmx_StartR			;Start DMX receive
	RETURN				;   Exit
SW_ST1	CLRF b_SW_TMR			;Push pressed, reset timeout timer
	BCF f_SW_TIM1			;Clear push timer toggle
	BCF f_SW_TIM2			;Clear push timeout timer flag
	BCF f_SW_TOUT			;Clear push timeout flag
	BTFSS b_SW_FAST,c_SW_BIT	;Check if buffer is already at level (160ms)
	GOTO SW_ST3			; No
SW_ST2	BTFSC f_RGB_RUN			; Yes, check if RGB is running (settings are available only with RGB off)
	RETURN				;  Running, exit
	BTFSC f_RGB_ON			;
	RETURN				;  Going on, exit
	BTFSC f_RGB_OFF			;
	RETURN				;  Going off, exit
	MOVF b_SW_SLOW,w		;  No, check if reached lower value (minimum long press time)
					;   NOTE: Slow counter increments in main loop every 100ms
	XORLW c_SW_SLOWL		;
	BZ SW_ST4			;   Yes
	MOVF b_SW_SLOW,w		;   No, check if reached higher value (maximum long press time)
	XORLW c_SW_SLOWH		;
	BZ SW_ST4			;    Yes
	RETURN				;    No, exit
SW_ST3	INCF b_SW_FAST,f		;fast buffer not at level, increase fast push counter
	BTFSS b_SW_FAST,c_SW_BIT	;Check if buffer is at level now (160ms)
	RETURN				; No, exit
	BSF f_SW_STAT			; Yes, set push flag
	BSF f_SW_FAST			; Set fast push flag
	MOVLF c_SW_SLIN,b_SW_SLOW	; Reload slow counter
	GOTO SW_ST2			; Proceed with slow push check
SW_ST4	BCF f_DMX_INH			;Disable DMX inhibit (DMX SLAVE, DEACTIVATE INHIBIT if long press)
	BANKSEL PIR3			;Bank 14
	BCF PIE3,RC1IE			;Disable serial receive IRQ
	BANKSEL PORTA			;Bank 0
	BCF f_SW_FAST			;Clear Fast press
	MOVLF c_SW_SLRP,b_SW_SLOW	;Slow counter at high level. reload slow counter for repeat mode
	BTFSC f_SW_SL0			;Slow counter at low level, check if slow press 0 (1-3 sec)
	GOTO SW_CNL1			; Yes
	BTFSC f_SW_SL1			; No, check if slow press 1 (3-5 sec)
	GOTO SW_CNL2			;  Yes
	BTFSC f_SW_SL2			;  No, check if slow press 2 (5-7 sec)
	GOTO SW_CNL3			;   Yes
	BTFSC f_SW_SL3			;   No, check if slow press 3 (7-9 sec)
	GOTO SW_CNL4			;    Yes
	BTFSC f_SW_SL4			;    No, check if slow press 4 (9-11 sec)
	GOTO SW_CNL5			;     Yes
	BTFSC f_SW_SL5			;    No, check if slow press 5 (>11 sec)
	GOTO SW_CNL6			;     Yes
	BSF f_SW_SL0			;     No, none of the above. Set Slow press 0
	RETURN
SW_CNL1	BCF f_SW_SL0			;Slow press 0, increase to Slow press 1
	BSF f_SW_SL1			;
	RETURN
SW_CNL2	BCF f_SW_SL1			;Slow press 1, increase to Slow press 2
	BSF f_SW_SL2			;
	RETURN
SW_CNL3	BCF f_SW_SL2			;Slow press 2, increase to Slow press 3
	BSF f_SW_SL3			;
	RETURN
SW_CNL4	BCF f_SW_SL3			;Slow press 3, increase to Slow press 4
	BSF f_SW_SL4			;
	RETURN
SW_CNL5	BCF f_SW_SL4			;Slow press 4, increase to Slow press 5
	BSF f_SW_SL5			;
	RETURN
SW_CNL6	DECF b_SW_SLOW,f		;Slow press 5, keep it here
	RETURN
