;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |          PWM routines for RGB frame          |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Type      : ASM                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________PWM init
;Initialize all PWM and RGB cycle related registers
;Modified registers:
;	system
;	w_RGB_CTRL
;	b_RGB_CNT
;	b_RGB_MODE
;	b_RGB_SPD
;Call method
;	CALL Pwm_Init
Pwm_Init:
	BANKSEL PWM3CON			;Bank 6
	MOVLF c_PWM3_CON,PWM3CON	;Set PWM3 module
	MOVLF c_PWM4_CON,PWM4CON	;Set PWM4 module
	MOVLF c_PWM5_CON,PWM5CON	;Set PWM5 module
	CLRF PWM3DCL			;Clear PWM registers (duty=0%)
	CLRF PWM3DCH			;
	CLRF PWM4DCL			;
	CLRF PWM4DCH			;
	CLRF PWM5DCL			;
	CLRF PWM5DCH			;
	BANKSEL T2PR			;Bank 5
	MOVLF c_PWM_PER,T2PR		;Set PWM period (period is the same for all PWM outputs)
	BSF T2CON,T2ON			;Starts Timer2 and PWM operations
	BANKSEL b_RGB_SPD		;Bank 0
	MOVLF c_RGB_FASTER,b_RGB_SPD	;Load faster delay to RGB cycle timer
	BSF f_RGB_SPD1			;Set speed faster (will be overwritten by SAF data, if available)
	BSF f_RGB_DONE			;Start with RGB OFF
	MOVLF 0xFF,b_PWM_RED		;Set Red 100% as first start color (will be overwritten by SAF data, if available)
	RETURN

;_______________________________________________________RGB demo test
;This function does a soft-ON and soft-OFF to the PWM registers, to
; avoid ugly flashes. Maximum On/Off time (depends on R,G,B values)
; is about 2.5sec
;Modified registers:
;	system
;	b_PWM_RED
;	b_PWM_GRN
;	b_PWM_BLU
;	b_RGB_CNT
;	b_RGB_CORR
;	R0H
;	R0L
;	R1H
;Call method
;	(Set RGB ON or set RGB OFF flag and clear RGB_RUN flag)
;	CALL RGB_OnOff
RGB_OnOff:
	BANKSEL w_RGB_CTRL		;Bank 0
	BTFSC f_DMX_INH			;Check if DMX inhibit is active
	GOTO RGBODX			; Yes, leave (DMX SLAVE, soft-on and off is disabled)
	BTFSC f_RGB_ON			;Check if going to ON
	GOTO RGBONR			; Yes
	BTFSC f_RGB_OFF			; No, check if going to OFF
	GOTO RGBOFR			;  Yes
	RETURN				;  No, nothing to do here
RGBONR	BSF f_RGB_END			;Going to ON, set End flag
	BTFSS f_RGB_MODE5		;Check if Mode 5 (fixed)
	GOTO RGBONRC			; No
	MOVFF b_PWM_RBK,R3H		; Yes, use fixed color setpoint
	GOTO RGBONRF			;
RGBONRC	MOVFF b_PWM_RED,R3H		;Use standard setpoint
RGBONRF	BZ RGBONG			;If RED PWM setpoint is zero, skip
	MOVF b_RGB_CORR,w		; Not zero, calculate actual RED PWM setpoint
	SUBWF R3H,w			;
	BANKSEL PWM3DCH			;Bank 6
	SUBWF PWM3DCH,w			;Check if RED PWM value is less than setpoint
	BGE RGBONG			; No
	INCF PWM3DCH,f			; Yes, increment value
	BANKSEL w_RGB_CTRL		; Bank 0
	BCF f_RGB_END			; Clear End flag
RGBONG	
	BANKSEL w_RGB_CTRL		;Bank 0
	BTFSS f_RGB_MODE5		;Check if Mode 5 (fixed)
	GOTO RGBONGC			; No
	MOVFF b_PWM_GBK,R3H		; Yes, use fixed color setpoint
	GOTO RGBONGF			;
RGBONGC	MOVFF b_PWM_GRN,R3H		;Use standard setpoint
RGBONGF	BZ RGBONB			;If GREEN PWM setpoint is zero, skip
	MOVF b_RGB_CORR,w		; Not zero, calculate actual GREEN PWM setpoint
	SUBWF R3H,w			;
	BANKSEL PWM4DCH			;Bank 6
	SUBWF PWM4DCH,w			;Check if GREEN PWM value is less than setpoint
	BGE RGBONB			; No
	INCF PWM4DCH,f			; Yes, increment value
	BANKSEL w_RGB_CTRL		; Bank 0
	BCF f_RGB_END			; Clear End flag
RGBONB	
	BANKSEL w_RGB_CTRL		;Bank 0
	BTFSS f_RGB_MODE5		;Check if Mode 5 (fixed)
	GOTO RGBONBC			; No
	MOVFF b_PWM_BBK,R3H		; Yes, use fixed color setpoint
	GOTO RGBONBF			;
RGBONBC	MOVFF b_PWM_BLU,R3H		;Use standard setpoint
RGBONBF	BZ RGBONZ			;If BLUE PWM setpoint is zero, skip
	MOVF b_RGB_CORR,w		; Not zero, calculate actual BLUE PWM setpoint
	SUBWF R3H,w			;
	BANKSEL PWM5DCH			;Bank 6
	SUBWF PWM5DCH,w			;Check if BLUE PWM value is less than setpoint
	BGE RGBONZ			; No
	INCF PWM5DCH,f			; Yes, increment value
	BANKSEL w_RGB_CTRL		; Bank 0
	BCF f_RGB_END			; Clear End flag
RGBONZ
	BANKSEL w_RGB_CTRL		;Bank 0
	BTFSS f_RGB_END			;Check if End flag is still set
	RETURN				; No, exit
	BCF f_RGB_END			; Yes, clear End flag
	BCF f_RGB_ON			; Clear ON flag
	BSF f_RGB_RUN			; Set RGB RUN flag (leaves control of RGB cycle to the main sub)
	RETURN
RGBOFR	BSF f_RGB_END			;Going to OFF, set End flag
	BANKSEL PWM3DCH			;Bank 6
	MOVF PWM3DCH,w			;Check if RED PWM value is zero
	BZ RGBOFG			; Yes
	DECF PWM3DCH,f			; No, decrement RED PWM value
	BANKSEL w_RGB_CTRL		; Bank 0
	BCF f_RGB_END			; Clear End flag
RGBOFG	
	BANKSEL PWM4DCH			;Bank 6
	MOVF PWM4DCH,w			;Check if GREEN PWM value is zero
	BZ RGBOFB			; Yes
	DECF PWM4DCH,f			; No, decrement GREEN PWM value
	BANKSEL w_RGB_CTRL		; Bank 0
	BCF f_RGB_END			; Clear End flag
RGBOFB	
	BANKSEL PWM5DCH			;Bank 6
	MOVF PWM5DCH,w			;Check if BLUE PWM value is zero
	BZ RGBOFZ			; Yes
	DECF PWM5DCH,f			; No, decrement BLUE PWM value
	BANKSEL w_RGB_CTRL		; Bank 0
	BCF f_RGB_END			; Clear End flag
RGBOFZ
	BANKSEL w_RGB_CTRL		;Bank 0
	BTFSS f_RGB_END			;Check if End flag is still set
	RETURN				; No, exit
	BCF f_RGB_END			; Yes, clear End flag
	BCF f_RGB_OFF			; Clear OFF flag
	RETURN
RGBODX	BCF f_RGB_END			;Clear End flag
	BCF f_RGB_OFF			;Clear OFF flag
	BCF f_RGB_ON			;Clear On flag
	RETURN


;_______________________________________________________RGB demo test
;This is only a demo to do a first lights-ON test of the circuit.
;Simple RGB cycle, without start/stop, no timing selection etc...
;Note: RGB_MODE is used here in a non-standard way
;	At power on, starts with all off, then:
;	0 RED ++ (no correction)
;	1 GREEN ++ (correction ++)
;	2 RED -- (correction --)
;	3 BLUE ++ (correction ++)
;	4 GREEN -- (correction --)
;	5 RED ++ (correction ++)
;	6 BLUE -- (correction --)
;	loop to 1
;Modified registers:
;	system
;	b_PWM_RED
;	b_PWM_GRN
;	b_PWM_BLU
;	b_RGB_MODE
;	b_RGB_CNT
;	b_RGB_CORR
;	R0H
;	R0L
;	R1H
;Call method
;	(Set mode in b_RGB_MODE)
;	(Load mode counter in b_RGB_CNT)
;       (Set RGB correction in b_RGB_CORR)
;	CALL RGB_Simple
;RGB_Simple:
;	BANKSEL b_RGB_MODE		;Bank 0
;	BTFSS f_RGB_RUN			;Check if RGB cycle is going
;	RETURN				; No, nothing to do here
;	BTFSC b_RGB_MODE,0		; Yes, check if first start
;	GOTO RGBS00			;  Yes, skip timing (go up quickly, about 10 sec to 100%)
;	DECFSZ b_RGB_SPD,f		;  No, decrement RGB cycle speed
;	RETURN				;    If not zero, exit
;	BTFSC f_RGB_SPD1		;    If zero, check which speed to load
;	MOVLW c_RGB_FASTER		;     Faster RGB cycle (60 sec)
;	BTFSC f_RGB_SPD2		;
;	MOVLW c_RGB_FAST		;     Fast RGB cycle (3 min)
;	BTFSC f_RGB_SPD3		;
;	MOVLW c_RGB_MEDIUM		;     Medium RGB cycle (7 min)
;	BTFSC f_RGB_SPD4		;
;	MOVLW c_RGB_SLOW		;     Slow RGB cycle (15 min)
;	BTFSC f_RGB_SPD5		;
;	MOVLW c_RGB_SLOWER		;     Slower RGB cycle (30 min)
;	MOVWF b_RGB_SPD			;
;	BTFSC b_RGB_MODE,1		;Not first start, check if GREEN goes UP
;	GOTO RGBS10			; Yep
;	BTFSC b_RGB_MODE,2		; Nope, check if RED goes DOWN
;	GOTO RGBS20			;  Yep
;	BTFSC b_RGB_MODE,3		;  Nope, check if BLUE goes UP
;	GOTO RGBS30			;   Yep
;	BTFSC b_RGB_MODE,4		;   Nope, check if GREEN goes DOWN
;	GOTO RGBS40			;    Yep
;	BTFSC b_RGB_MODE,5		;    Nope, check if RED goes UP
;	GOTO RGBS50			;     Yep
;	BTFSC b_RGB_MODE,6		;     Nope, check if BLUE goes DOWN
;	GOTO RGBS60			;      Yep
;					;      Nope. If I'm here, something went wrong. Don't care in a demo, just restart from the beginning
;	MOVLF 0x01,b_RGB_MODE		;       Start with mode 0 - Used only for demo test
;	MOVLF 0x03,b_RGB_CNT		;       Load RGB mode counter - TBD, this way only for demo test
;	CLRF b_RGB_CORR			;       Clear RGB duty correction
;	CLRF b_PWM_RED			;       Clear RED pwm duty
;	CLRF b_PWM_GRN			;       Clear RED pwm duty
;	CLRF b_PWM_BLU			;       Clear RED pwm duty
;RGBS00	INCFSZ b_PWM_RED,f		;Increase RED
;	GOTO RGBS01			; Not reached max
;	MOVLF 0xFF,b_PWM_RED		; Reached max, fix value
;	BCF b_RGB_MODE,0		; Set next operation increase GREEN
;	BSF b_RGB_MODE,1		;
;RGBS01	MOVFF b_PWM_RED,R0H		;Copy RED duty counter to temp
;	CLRF R0L			;Clear GREEN duty temp
;	CLRF R1H			;Clear BLUE duty temp
;	GOTO RGBSZZ
;RGBS10	INCFSZ b_PWM_GRN,f		;Increase GREEN
;	GOTO RGBS11			; Not reached max
;	MOVLF 0xFF,b_PWM_GRN		; Reached max, fix value
;	BCF b_RGB_MODE,1		; Reached min, set next operation decrease RED
;	BSF b_RGB_MODE,2		;
;	MOVLF 0x02,b_RGB_CNT		; Reload cycle counter
;	MOVLF 0x80, b_RGB_CORR		;Reset duty correction
;	GOTO RGBS22			;
;RGBS11	DECFSZ b_RGB_CNT,f		;Decrement cycle counter. Zero?
;	GOTO RGBS22			; No
;	MOVLF 0x02,b_RGB_CNT		; Yes, reload cycle counter
;	INCF b_RGB_CORR,f		; Increase duty correction
;	GOTO RGBS22			;
;RGBS20	DECFSZ b_PWM_RED,f		;Decrease RED
;	GOTO RGBS21			; Not reached min
;	BCF b_RGB_MODE,2		; Reached min, set next operation increase BLUE
;	BSF b_RGB_MODE,3		;
;	MOVLF 0x02,b_RGB_CNT		; Reload cycle counter
;	CLRF b_RGB_CORR			;Reset duty correction
;	GOTO RGBS22			;
;RGBS21	DECFSZ b_RGB_CNT,f		;Decrement cycle counter. Zero?
;	GOTO RGBS22			; No
;	MOVLF 0x02,b_RGB_CNT		; Yes, reload cycle counter
;	DECF b_RGB_CORR,f		; Decrease duty correction
;RGBS22	MOVFF b_PWM_RED,R0H		;Copy RED duty counter to temp
;	MOVF b_RGB_CORR,w		;Copy duty correction to w
;	SUBWF R0H,f			;Subtract duty correction from RED duty counter
;	MOVFF b_PWM_GRN,R0L		;Copy GREEN duty counter to temp
;	MOVF b_RGB_CORR,w		;Copy duty correction to w
;	SUBWF R0L,f			;Subtract duty correction from GREEN duty counter
;	CLRF R1H			;Clear BLUE duty temp
;	GOTO RGBSZZ
;RGBS30	INCFSZ b_PWM_BLU,f		;Increase BLUE
;	GOTO RGBS31			; Not reached max
;	MOVLF 0xFF,b_PWM_BLU		; Reached max, fix value
;	BCF b_RGB_MODE,3		; Reached min, set next operation decrease GREEN
;	BSF b_RGB_MODE,4		;
;	MOVLF 0x02,b_RGB_CNT		; Reload cycle counter
;	MOVLF 0x80, b_RGB_CORR		;Reset duty correction
;	GOTO RGBS42			;
;RGBS31	DECFSZ b_RGB_CNT,f		;Decrement cycle counter. Zero?
;	GOTO RGBS42			; No
;	MOVLF 0x02,b_RGB_CNT		; Yes, reload cycle counter
;	INCF b_RGB_CORR,f		; Increase duty correction
;	GOTO RGBS42			;
;RGBS40	DECFSZ b_PWM_GRN,f		;Decrease GREEN
;	GOTO RGBS41			; Not reached min
;	BCF b_RGB_MODE,4		; Reached min, set next operation increase RED
;	BSF b_RGB_MODE,5		;
;	MOVLF 0x02,b_RGB_CNT		; Reload cycle counter
;	CLRF b_RGB_CORR			;Reset duty correction
;	GOTO RGBS42			;
;RGBS41	DECFSZ b_RGB_CNT,f		;Decrement cycle counter. Zero?
;	GOTO RGBS42			; No
;	MOVLF 0x02,b_RGB_CNT		; Yes, reload cycle counter
;	DECF b_RGB_CORR,f		; Decrease duty correction
;RGBS42	CLRF R0H			;Clear RED duty temp
;	MOVFF b_PWM_GRN,R0L		;Copy GREEN duty counter to temp
;	MOVF b_RGB_CORR,w		;Copy duty correction to w
;	SUBWF R0L,f			;Subtract duty correction from GREEN duty counter
;	MOVFF b_PWM_BLU,R1H		;Copy BLUE duty counter to temp
;	MOVF b_RGB_CORR,w		;Copy duty correction to w
;	SUBWF R1H,f			;Subtract duty correction from BLUE duty counter
;	GOTO RGBSZZ
;RGBS50	INCFSZ b_PWM_RED,f		;Increase RED
;	GOTO RGBS51			; Not reached max
;	MOVLF 0xFF,b_PWM_RED		; Reached max, fix value
;	BCF b_RGB_MODE,5		; Reached min, set next operation decrease BLUE
;	BSF b_RGB_MODE,6		;
;	MOVLF 0x02,b_RGB_CNT		; Reload cycle counter
;	MOVLF 0x80, b_RGB_CORR		;Reset duty correction
;	GOTO RGBS62			;
;RGBS51	DECFSZ b_RGB_CNT,f		;Decrement cycle counter. Zero?
;	GOTO RGBS62			; No
;	MOVLF 0x02,b_RGB_CNT		; Yes, reload cycle counter
;	INCF b_RGB_CORR,f		; Increase duty correction
;	GOTO RGBS62			;
;RGBS60	DECFSZ b_PWM_BLU,f		;Decrease BLUE
;	GOTO RGBS61			; Not reached min
;	BCF b_RGB_MODE,6		; Reached min, set next operation increase GREEN
;	BSF b_RGB_MODE,1		;
;	MOVLF 0x02,b_RGB_CNT		; Reload cycle counter
;	CLRF b_RGB_CORR			;Reset duty correction
;	GOTO RGBS62			;
;RGBS61	DECFSZ b_RGB_CNT,f		;Decrement cycle counter. Zero?
;	GOTO RGBS62			; No
;	MOVLF 0x02,b_RGB_CNT		; Yes, reload cycle counter
;	DECF b_RGB_CORR,f		; Decrease duty correction
;RGBS62	MOVFF b_PWM_RED,R0H		;Copy RED duty counter to temp
;	MOVF b_RGB_CORR,w		;Copy duty correction to w
;	SUBWF R0H,f			;Subtract duty correction from RED duty counter
;	CLRF R0L			;Clear GREEN duty temp
;	MOVFF b_PWM_BLU,R1H		;Copy BLUE duty counter to temp
;	MOVF b_RGB_CORR,w		;Copy duty correction to w
;	SUBWF R1H,f			;Subtract duty correction from BLUE duty counter
;RGBSZZ
;	BANKSEL PWM3DCH			;Bank 6
;	MOVFF R0H,PWM3DCH		;load RED duty value in timer
;	MOVFF R0L,PWM4DCH		;load GREEN duty value in timer
;	MOVFF R1H,PWM5DCH		;load BLUE duty value in timer
;	CLRF PWM3DCL			;
;	CLRF PWM4DCL			;
;	CLRF PWM5DCL			;
;	BANKSEL PORTA			;Bank 0
;	RETURN
	

;_______________________________________________________RGB full colors
;This is the main sub for changing colors. It includes all the color changing
; modes that are available to the system
;At power on, starts with all off, then - unless specified in EEPROM,
; starts with a simple RGB cycle. All modifications can be made through
; the pushbutton. This sub needs the RGB management sub to set the right params
;Modified registers:
;	system
;	b_PWM_RED
;	b_PWM_GRN
;	b_PWM_BLU
;	b_RGB_CNT
;	b_RGB_CORR
;	w_RGB_CTRL
;	R0H
;	R1H
;	R1L
;	R2H
;Call method
;	CALL RGB_Manager
;	CALL RGB_Cycle
RGB_Cycle:
    	BANKSEL w_RGB_CTRL		;Bank 0
	BTFSC f_DMX_INH			;Check if DMX inhibit is active
	GOTO RC_DMX			; Yes, simply copy the DMX received values
	BTFSC f_RGB_ON			;Check if turning On
	RETURN				; Yes, nothing to do
	BTFSC f_RGB_OFF			; No, check if turning Off
	RETURN				;  Yes, nothing to do
	BTFSS f_RGB_RUN			;  No, check if running
	RETURN				;   No, nothing to do
	BTFSC f_RGB_DONE		;   Yes, check RGB done flag
	RETURN				;    Set, nothing to do, exit
	DECFSZ b_RGB_SPD,f		;    Clear, decrement RGB cycle speed
	RETURN				;     Not zero, exit
	BTFSC f_RGB_SPD1		;     Zero, check which speed to load
	MOVLW c_RGB_FASTER		;      Faster RGB cycle (60 sec)
	BTFSC f_RGB_SPD2		;
	MOVLW c_RGB_FAST		;      Fast RGB cycle (3 min)
	BTFSC f_RGB_SPD3		;
	MOVLW c_RGB_MEDIUM		;      Medium RGB cycle (7 min)
	BTFSC f_RGB_SPD4		;
	MOVLW c_RGB_SLOW		;      Slow RGB cycle (15 min)
	BTFSC f_RGB_SPD5		;
	MOVLW c_RGB_SLOWER		;      Slower RGB cycle (30 min)
	MOVWF b_RGB_SPD			;
	MOVF b_RGB_MODE,w		;Check RGB mode
	BZ RC_ERR			; 0, this is an error
	MOVWF R0H
	DECF R0H,f
	BZ RC_01			;R+
	DECF R0H,f
	BZ RC_02			;R+ G
	DECF R0H,f
	BZ RC_03			;R+    B
	DECF R0H,f
	BZ RC_04			;R+ G+
	DECF R0H,f
	BZ RC_05			;R+ G+ B
	DECF R0H,f
	BZ RC_06			;R+    B+
	DECF R0H,f
	BZ RC_07			;R+ G  B+
	DECF R0H,f
	BZ RC_08			;R-
	DECF R0H,f
	BZ RC_09			;R- G
	DECF R0H,f
	BZ RC_10			;R-    B
	DECF R0H,f
	BZ RC_11			;R- G-
	DECF R0H,f
	BZ RC_12			;R- G- B
	DECF R0H,f
	BZ RC_13			;R-    B-
	DECF R0H,f
	BZ RC_14			;R- G  B-
	DECF R0H,f
	BZ RC_15			;   G+
	DECF R0H,f
	BZ RC_16			;R  G+
	DECF R0H,f
	BZ RC_17			;   G+ B
	DECF R0H,f
	BZ RC_18			;   G+ B+
	DECF R0H,f
	BZ RC_19			;R  G+ B+
	DECF R0H,f
	BZ RC_20			;   G-
	DECF R0H,f
	BZ RC_21			;R  G-
	DECF R0H,f
	BZ RC_22			;   G- B
	DECF R0H,f
	BZ RC_23			;   G- B-
	DECF R0H,f
	BZ RC_24			;R  G- B-
	DECF R0H,f
	BZ RC_25			;      B+
	DECF R0H,f
	BZ RC_26			;R     B+
	DECF R0H,f
	BZ RC_27			;   G  B+
	DECF R0H,f
	BZ RC_28			;      B-
	DECF R0H,f
	BZ RC_29			;R     B-
	DECF R0H,f
	BZ RC_30			;   G  B-
	DECF R0H,f			;
	BZ RC_31			;Fixed colors
	GOTO RC_ERR			;RGB Mode is greater than 31, error
RC_01	CLRF b_PWM_GRN			;Mode 1, R+, clear G and B
	CLRF b_PWM_BLU			;
	INCFSZ b_PWM_RED,f		;R++
	GOTO RC_011			; Not zero, copy set points
	MOVLF 0xFF,b_PWM_RED		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
RC_011	MOVFF b_PWM_RED,R1H		;Copy setpoints to temp registers
	CLRF R1L			;
	CLRF R2H			;
	GOTO RC_END			;Set PWM and exit
RC_02	MOVLF 0xFF,b_PWM_GRN		;Mode 2, (R+ G), set G and clear B
	CLRF b_PWM_BLU			;
	INCFSZ b_PWM_RED,f		;R++
	GOTO RC_021			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_RED		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
RC_021	DECFSZ b_RGB_CNT,f		;Decrement color correction timer
	GOTO RC_022			; Not zero, copy set points
	MOVLF c_TMR_COR2,b_RGB_CNT	; Zero, reset timer
	INCF b_RGB_CORR,f		; Increment color correction
RC_022	MOVF b_RGB_CORR,w		;Subtract duty correction from RED duty counter
	SUBWF b_PWM_RED,w		;
	MOVWF R1H			;Copy corrected setpoint to temp register
	MOVF b_RGB_CORR,w		;Subtract duty correction from GREEN duty counter
	SUBWF b_PWM_GRN,w		;
	MOVWF R1L			;Copy corrected setpoint to temp register
	CLRF R2H			;BLUE=0
	GOTO RC_END			;Set PWM and exit
RC_03	MOVLF 0xFF,b_PWM_BLU		;Mode 3, (R+ B), set B and clear G
	CLRF b_PWM_GRN			;
	INCFSZ b_PWM_RED,f		;R++
	GOTO RC_031			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_RED		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
RC_031	DECFSZ b_RGB_CNT,f		;Decrement color correction timer
	GOTO RC_032			; Not zero, copy set points
	MOVLF c_TMR_COR2,b_RGB_CNT	; Zero, reset timer
	INCF b_RGB_CORR,f		; Increment color correction
RC_032	MOVF b_RGB_CORR,w		;Subtract duty correction from RED duty counter
	SUBWF b_PWM_RED,w		;
	MOVWF R1H			;Copy corrected setpoint to temp register
	MOVF b_RGB_CORR,w		;Subtract duty correction from BLUE duty counter
	SUBWF b_PWM_BLU,w		;
	MOVWF R2H			;Copy corrected setpoint to temp register
	CLRF R1L			;GREEN=0
	GOTO RC_END			;Set PWM and exit
RC_04	CLRF b_PWM_BLU			;Mode 4, (R+ G+), clear B
	INCF b_PWM_RED,f		;R++
	INCFSZ b_PWM_GRN,f		;G++
	GOTO RC_021			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_RED		; Zero, reached end, reset to 255
	MOVWF b_PWM_GRN			;
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
	GOTO RC_021			; Correct set points
RC_05	MOVLF 0xFF,b_PWM_BLU		;Mode 5, (R+ G+ B), set B
	INCF b_PWM_RED,f		;R++
	INCFSZ b_PWM_GRN,f		;G++
	GOTO RC_051			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_RED		; Zero, reached end, reset to 255
	MOVWF b_PWM_GRN			;
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
RC_051	DECFSZ b_RGB_CNT,f		;Decrement color correction timer1
	GOTO RC_053			; Not zero, copy set points
	INCF b_RGB_CORR,f		; Zero, increment color correction
	DECFSZ b_RGB_CNT2,f		; Decrement color correction timer2
	GOTO RC_052			;  Not zero
	INCF b_RGB_CORR,f		;  Zero, increment color correction
	MOVLF c_TMR_CORR,b_RGB_CNT2	;  Reset color correction timer2
RC_052	MOVFF b_RGB_CNT2,b_RGB_CNT	;Copy color correction timer2 value into timer1
RC_053	MOVF b_RGB_CORR,w		;Subtract duty correction from RED duty counter
	SUBWF b_PWM_RED,w		;
	MOVWF R1H			;Copy corrected setpoint to temp register
	MOVF b_RGB_CORR,w		;Subtract duty correction from GREEN duty counter
	SUBWF b_PWM_GRN,w		;
	MOVWF R1L			;Copy corrected setpoint to temp register
	MOVF b_RGB_CORR,w		;Subtract duty correction from BLUE duty counter
	SUBWF b_PWM_BLU,w		;
	MOVWF R2H			;Copy corrected setpoint to temp register
	GOTO RC_END			;Set PWM and exit
RC_06	CLRF b_PWM_GRN			;Mode 6, (R+ B+), clear R
	INCF b_PWM_RED,f		;R++
	INCFSZ b_PWM_BLU,f		;B++
	GOTO RC_031			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_RED		; Zero, reached end, reset to 255
	MOVWF b_PWM_BLU			;
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
	GOTO RC_031			; Correct set points
RC_07	MOVLF 0xFF,b_PWM_GRN		;Mode 7, (R+ G B+), set G
	INCF b_PWM_RED,f		;R++
	INCFSZ b_PWM_BLU,f		;B++
	GOTO RC_051			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_RED		; Zero, reached end, reset to 255
	MOVWF b_PWM_BLU			;
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
	GOTO RC_051			; Correct set points
RC_08	CLRF b_PWM_GRN			;Mode 8, R-, clear G and B
	CLRF b_PWM_BLU			;
	DECFSZ b_PWM_RED,f		;R--
	GOTO RC_011			; Not zero, copy set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_011			; Copy set points
RC_09	MOVLF 0xFF,b_PWM_GRN		;Mode 9, (R- G), set G and clear B
	CLRF b_PWM_BLU			;
	DECFSZ b_PWM_RED,f		;R--
	GOTO RC_091			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
RC_091	DECFSZ b_RGB_CNT,f		;Decrement color correction timer
	GOTO RC_022			; Not zero, copy set points
	MOVLF c_TMR_COR2,b_RGB_CNT	; Zero, reset timer
	DECF b_RGB_CORR,f		; Decrement color correction
	MOVLW 0xFF			; Chech if gone around to 255
	XORWF b_RGB_CORR,w		;
	BNZ RC_022			;  No
	CLRF b_RGB_CORR			;  Yes, reset to 0
	GOTO RC_022			;
RC_10	MOVLF 0xFF,b_PWM_BLU		;Mode 10, (R- B), set B and clear G
	CLRF b_PWM_GRN			;
	DECFSZ b_PWM_RED,f		;R--
	GOTO RC_101			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
RC_101	DECFSZ b_RGB_CNT,f		;Decrement color correction timer
	GOTO RC_032			; Not zero, copy set points
	MOVLF c_TMR_COR2,b_RGB_CNT	; Zero, reset timer
	DECF b_RGB_CORR,f		; Decrement color correction
	MOVLW 0xFF			; Chech if gone around to 255
	XORWF b_RGB_CORR,w		;
	BNZ RC_032			;  No
	CLRF b_RGB_CORR			;  Yes, reset to 0
	GOTO RC_032
RC_11	CLRF b_PWM_BLU			;Mode 11, (R- G-), clear B
	DECF b_PWM_RED,f		;R--
	DECFSZ b_PWM_GRN,f		;G--
	GOTO RC_091			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_091			; Correct set points
RC_12	MOVLF 0xFF,b_PWM_BLU		;Mode 12, (R- G- B), set B
	DECF b_PWM_RED,f		;R--
	DECFSZ b_PWM_GRN,f		;G--
	GOTO RC_121			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
RC_121	DECFSZ b_RGB_CNT,f		;Decrement color correction timer1
	GOTO RC_053			; Not zero, copy set points
	DECF b_RGB_CORR,f		; Zero, decrement color correction
	DECFSZ b_RGB_CNT2,f		; Decrement color correction timer2
	GOTO RC_123			;  Not zero
	DECF b_RGB_CORR,f		;  Zero, decrement color correction
	MOVLW 0xFF			;  Check if gone around to 255
	XORWF b_RGB_CORR,w		;
	BNZ RC_122			;  No
	CLRF b_RGB_CORR			;  Yes, reset to 0
RC_122	MOVLF c_TMR_CORR,b_RGB_CNT2	;  Reset color correction timer2
RC_123	MOVFF b_RGB_CNT2,b_RGB_CNT	;Copy color correction timer2 value into timer1
	GOTO RC_053
RC_13	CLRF b_PWM_GRN			;Mode 13, (R- B-), clear G
	DECF b_PWM_RED,f		;R--
	DECFSZ b_PWM_BLU,f		;B--
	GOTO RC_101			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_101			; Correct set points
RC_14	MOVLF 0xFF,b_PWM_GRN		;Mode 14, (R- G B-), set G
	DECF b_PWM_RED,f		;R--
	DECFSZ b_PWM_BLU,f		;B--
	GOTO RC_121			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_121			; Correct set points
RC_15	CLRF b_PWM_RED			;Mode 15, G+, clear R and B
	CLRF b_PWM_BLU			;
	INCFSZ b_PWM_GRN,f		;G++
	GOTO RC_151			; Not zero, copy set points
	MOVLF 0xFF,b_PWM_GRN		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
RC_151	CLRF R1H			;Copy setpoints to temp registers
	MOVFF b_PWM_GRN,R1L		;
	CLRF R2H			;
	GOTO RC_END			;Set PWM and exit
RC_16	MOVLF 0xFF,b_PWM_RED		;Mode 16, (R G+), set R and clear B
	CLRF b_PWM_BLU			;
	INCFSZ b_PWM_GRN,f		;G++
	GOTO RC_021			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_GRN		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
	GOTO RC_021			; Correct set points
RC_17	MOVLF 0xFF,b_PWM_BLU		;Mode 17, (G+ B), set B and clear R
	CLRF b_PWM_RED			;
	INCFSZ b_PWM_GRN,f		;G++
	GOTO RC_171			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_GRN		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
RC_171	DECFSZ b_RGB_CNT,f		;Decrement color correction timer
	GOTO RC_172			; Not zero, copy set points
	MOVLF c_TMR_COR2,b_RGB_CNT	; Zero, reset timer
	INCF b_RGB_CORR,f		; Increment color correction
RC_172	MOVF b_RGB_CORR,w		;Subtract duty correction from GREEN duty counter
	SUBWF b_PWM_GRN,w		;
	MOVWF R1L			;Copy corrected setpoint to temp register
	MOVF b_RGB_CORR,w		;Subtract duty correction from BLUE duty counter
	SUBWF b_PWM_BLU,w		;
	MOVWF R2H			;Copy corrected setpoint to temp register
	CLRF R1H			;RED=0
	GOTO RC_END			;Set PWM and exit
RC_18	CLRF b_PWM_RED			;Mode 20, (G+ B+), clear R
	INCF b_PWM_GRN,f		;G++
	INCFSZ b_PWM_BLU,f		;B++
	GOTO RC_171			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_GRN		; Zero, reached end, reset to 255
	MOVWF b_PWM_BLU			;
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
	GOTO RC_171			; Correct set points
RC_19	MOVLF 0xFF,b_PWM_RED		;Mode 21, (R G+ B+), set R
	INCF b_PWM_GRN,f		;G++
	INCFSZ b_PWM_BLU,f		;B++
	GOTO RC_051			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_GRN		; Zero, reached end, reset to 255
	MOVWF b_PWM_BLU			;
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
	GOTO RC_051			; Correct set points
RC_20	CLRF b_PWM_RED			;Mode 22, G-, clear R and B
	CLRF b_PWM_BLU			;
	DECFSZ b_PWM_GRN,f		;G--
	GOTO RC_151			; Not zero, copy set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_151			; Copy set points
RC_21	MOVLF 0xFF,b_PWM_RED		;Mode 23, (R G-), set R and clear B
	CLRF b_PWM_BLU			;
	DECFSZ b_PWM_GRN,f		;G--
	GOTO RC_091			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_091			; Correct set points
RC_22	MOVLF 0xFF,b_PWM_BLU		;Mode 24, (G- B), set B and clear R
	CLRF b_PWM_RED			;
	DECFSZ b_PWM_GRN,f		;G--
	GOTO RC_221			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
RC_221	DECFSZ b_RGB_CNT,f		;Decrement color correction timer
	GOTO RC_172			; Not zero, copy set points
	MOVLF c_TMR_COR2,b_RGB_CNT	; Zero, reset timer
	DECF b_RGB_CORR,f		; Decrement color correction
	MOVLW 0xFF			; Chech if gone around to 255
	XORWF b_RGB_CORR,w		;
	BNZ RC_172			;  No
	CLRF b_RGB_CORR			;  Yes, reset to 0
	GOTO RC_172			;
RC_23	CLRF b_PWM_RED			;Mode 27, (G- B-), clear R
	DECF b_PWM_GRN,f		;G--
	DECFSZ b_PWM_BLU,f		;B--
	GOTO RC_221			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_221			; Correct set points
RC_24	MOVLF 0xFF,b_PWM_RED		;Mode 12, (R G- B-), set R
	DECF b_PWM_GRN,f		;G--
	DECFSZ b_PWM_BLU,f		;B--
	GOTO RC_121			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_121			; Correct set points
RC_25	CLRF b_PWM_RED			;Mode 29, B+, clear R and G
	CLRF b_PWM_GRN			;
	INCFSZ b_PWM_BLU,f		;B++
	GOTO RC_251			; Not zero, copy set points
	MOVLF 0xFF,b_PWM_BLU		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
RC_251	MOVFF b_PWM_BLU,R2H		;Copy setpoints to temp registers
	CLRF R1H			;
	CLRF R1L			;
	GOTO RC_END			;Set PWM and exit
RC_26	MOVLF 0xFF,b_PWM_RED		;Mode 30, (R B+), set R and clear G
	CLRF b_PWM_GRN			;
	INCFSZ b_PWM_BLU,f		;B++
	GOTO RC_031			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_BLU		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
	GOTO RC_031			; Correct set points
RC_27	MOVLF 0xFF,b_PWM_GRN		;Mode 31, (G B+), set G and clear R
	CLRF b_PWM_RED			;
	INCFSZ b_PWM_BLU,f		;B++
	GOTO RC_171			; Not zero, correct set points
	MOVLF 0xFF,b_PWM_BLU		; Zero, reached end, reset to 255
	BSF f_RGB_DONE			; Set Done flag (end of cycle)
	GOTO RC_171			; Correct set points
RC_28	CLRF b_PWM_RED			;Mode 36, B-, clear R and G
	CLRF b_PWM_GRN			;
	DECFSZ b_PWM_BLU,f		;B--
	GOTO RC_251			; Not zero, copy set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_251			; Copy set points
RC_29	MOVLF 0xFF,b_PWM_RED		;Mode 37, (R B-), set R and clear G
	CLRF b_PWM_GRN			;
	DECFSZ b_PWM_BLU,f		;B--
	GOTO RC_101			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_101			; Correct set points
RC_30	MOVLF 0xFF,b_PWM_GRN		;Mode 38, (G B-), set G and clear R
	CLRF b_PWM_RED			;
	DECFSZ b_PWM_BLU,f		;B--
	GOTO RC_221			; Not zero, correct set points
	BSF f_RGB_DONE			; Zero, set Done flag (end of cycle)
	GOTO RC_221			; Correct set points
RC_31	MOVFF b_PWM_RBK,R1H		;Fixed colors, reload setpoints from backup
	MOVFF b_PWM_GBK,R1L		;
	MOVFF b_PWM_BBK,R2H		;
	BSF f_RGB_DONE			;Set RGB done flag
	GOTO RC_END			;
RC_ERR	BSF f_RGB_ERR			;Error, set flag
	BSF f_RGB_DONE			;Set RGB done flag
	CLRF R1H			;Set all PWM to 0
	CLRF R1L			;
	CLRF R2H			;
RC_END
	BANKSEL PWM3DCH			;Bank 6
	MOVFF R1H,PWM3DCH		;load RED duty value in timer
	MOVFF R1L,PWM4DCH		;load GREEN duty value in timer
	MOVFF R2H,PWM5DCH		;load BLUE duty value in timer
	CLRF PWM3DCL			;
	CLRF PWM4DCL			;
	CLRF PWM5DCL			;
	BANKSEL PORTA			;Bank 0
	RETURN
;DMX SLAVE, RGB is simply taken from bus
RC_DMX	MOVFF b_DMX_RED,R1H		;load RED duty value in temp
	MOVFF b_DMX_GRN,R1L		;load GREEN duty value in temp
	MOVFF b_DMX_BLU,R2H		;load BLUE duty value in temp
	GOTO RC_END
