;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |        System routines for RGB frame         |
; |______________________________________________|
; |   Src File                                   |
; |______________________________________________|
; |   Type      : ASM                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________System init
;Initialize all system related registers not managed elsewhere, and
; clear all RAM
;Modified registers:
;	system
;	RAM (all)
;Call method
;	CALL Sys_Init
Sys_Init:
	BANKSEL PMD0			;Bank 15
	MOVLF c_PMD0,PMD0		;Peripherals enable/disable
	MOVLF c_PMD1,PMD1		;
	MOVLF c_PMD2,PMD2		;
	MOVLF c_PMD3,PMD3		;
	MOVLF c_PMD4,PMD4		;
	MOVLF c_PMD5,PMD5		;
	MOVLF 0x20,FSR0H		;Load HIGH address of linear memory area
	CLRF FSR0L			;Load LOW address
SYS_IN1	CLRF INDF0			;Clear memory byte
	INCF FSR0L,f			;Increment position
	BTFSC STATUS,Z			;Check if zero reached
	INCF FSR0H,f			;YES, Increment HIGH address
        BTFSS FSR0H,c_MAXRAM		;Check if reached RAM limit
	GOTO SYS_IN1			;NO, keep going
	BANKSEL PORTA			;Bank 0
	MOVLF c_MENU_LVL0,b_MENU_LVL	;Init menu level
	BSF f_RGB_MODE0			;Init RGB mode
;System will start with step 0 and mode (R+)
	MOVLF 0x01,b_RGB_BLINK		;Init blink timer
	MOVLF c_RGB_REDR,b_PWM_RBK	;Set initial fixed color as 100% RED (will be overwritten by SAF data, if available)
	MOVLF c_DMXA_DLY,b_DMX_DLYCNT	;Init delay counter
	RETURN

;_______________________________________________________System Menu
;Manages the system status based on the pushbutton input
;Modified registers:
;	system
;	b_MENU_LVL
;	b_E2_FLAGS
;	w_DMX_ADDR
;	w_SW_STAT
;Call method
;	CALL Sys_Menu
Sys_Menu:
	BANKSEL w_SW_STAT		;Bank 0
	BTFSC f_RGB_ON			;Check if starting
	RETURN				; Yes, exit
	BTFSC f_RGB_OFF			; No, check if stopping
	RETURN				;  Yes, exit
	BTFSS f_SW_TOUT			;Check if push timeout
	GOTO SM_STS			; No
	MOVLF c_MENU_LVL0,b_MENU_LVL	;  Yes, set menu level 0
	MOVF w_DMX_ADDR,w		;  Check if DMX address is bigger than 255
	BZ SM_DX3			;   No
	DECFSZ w_DMX_ADDR,w		;   Yes, check if it is bigger than 511
	GOTO SM_DX2			;    Yes
SM_DX1	MOVF w_DMX_ADDR+1,w		;    No, check if >509 (chould not be, check just in case)
	ADDLW 0x02			;
	BNC SM_DX3			;     No
SM_DX2	MOVLF 0x01,w_DMX_ADDR		;     Yes, force address to 509
	MOVLF 0xFD,w_DMX_ADDR+1
	BSF PCLATH,3			;Page 1
	CALL Dmx_Addr_BCD		;Convert new DMX address into hundreds,
					; tens and units, and reload temp registers
	BCF PCLATH,3			;Page 0
	GOTO SM_DX4
SM_DX3  MOVF w_DMX_ADDR,w		;Address is smaller than 509, check if it is 0
	BNZ SM_DX4			; No, >255
	MOVF w_DMX_ADDR+1,w		; <256, check LSB
	BNZ SM_DX4			;  >0
	MOVLF 0x01,w_DMX_ADDR+1		;  =0, force address to 001
	BSF PCLATH,3			;Page 1
	CALL Dmx_Addr_BCD		;Convert new DMX address into hundreds,
					; tens and units, and reload temp registers
	BCF PCLATH,3			;Page 0
SM_DX4  BCF f_SW_TOUT			;Clear pushbutton timeout flag
	CLRF w_SW_STAT			;Clear fast and slow push, status and status change flags
	BCF f_SW_CHG			;
	BTFSC f_RGB_RUN			; Check if RGB is running
	RETURN				;  Yes, end
	BTFSC f_RGB_MODE5		;  No, check if Mode 5 (static colors)
	RETURN				;   Yes, end
	BANKSEL PWM3DCH			;Bank 6
	CLRF PWM3DCH			;   No, clear RED duty value
	CLRF PWM4DCH			;   Clear GREEN duty value
	CLRF PWM5DCH			;   Clear BLUE duty value
	CLRF PWM3DCL			;
	CLRF PWM4DCL			;
	CLRF PWM5DCL			;
	BANKSEL PORTA			;Bank 0
	RETURN
SM_STS	BTFSS f_SW_CHG			;Check if push status change is set
	RETURN				; No, nothing to do
	BTFSC f_MENU_LVL0		; Yes, check menu level
	GOTO SM_L0			; Level 0
	BTFSC f_MENU_LVL1		;
	GOTO SM_L1			; Level 1
	BTFSC f_MENU_LVL2		;
	GOTO SM_L2			; Level 2
	BTFSC f_MENU_LVL3		;
	GOTO SM_L3			; Level 3
	BTFSC f_MENU_LVL4		;
	GOTO SM_L4			; Level 4
	BTFSC f_MENU_LVL5		;
	GOTO SM_L5			; Level 5
	BTFSC f_MENU_LVL6		;
	GOTO SM_L6			; Level 6
	BTFSC f_MENU_LVL7		;
	GOTO SM_L7			; Level 7
	MOVLF c_MENU_LVL0,b_MENU_LVL	; No level selected, reset to 0
	GOTO SM_END			; End
SM_L0	BTFSC f_SW_FAST			;Level 0, check if fast push
	GOTO SM_L0F			; Yes
SM_L0S0	BTFSS f_SW_SL0			; No, check if slow push 0
	GOTO SM_L0S1			;  No
	MOVLF c_MENU_LVL1,b_MENU_LVL	;  Yes, set Menu level 1 (RGB speed)
	BCF f_RGB_RUN			;  Stop standard RGB cycle
	GOTO SM_END
SM_L0S1	BTFSS f_SW_SL1			;Check if slow push 1
	GOTO SM_L0S2			; No
	MOVLF c_MENU_LVL2,b_MENU_LVL	; Yes, set Menu level 2 (RGB mode)
	BCF f_RGB_RUN			; Stop standard RGB cycle
	GOTO SM_END
SM_L0S2	BTFSS f_SW_SL2			;Check if slow push 2
	GOTO SM_L0S3			; No
	MOVLF c_MENU_LVL3,b_MENU_LVL	; Yes, set Menu level 3 (Fixed colors)
	BSF f_RGB_RUN			; Enable RGB cycle
	ANDLF c_MODE_MASK,b_MENU_FLAGS	; Clear modes
	BSF f_RGB_MODE5			; Set mode 5
	BSF f_RGB_DONE			; Set step finished (force change to next step)
	CLRF b_RGB_CORR			; Clear color correction (needed for soft-on)
	MOVLF .31,b_RGB_MODE		; Set mode (fixed colors)
	GOTO SM_END
SM_L0S3	BTFSS f_SW_SL3			;Check if slow push 3
	GOTO SM_L0S4			; No
	MOVLF c_MENU_LVL4,b_MENU_LVL	; Yes, set Menu level 4 (DMX mode)
	BCF f_RGB_RUN			; Stop standard RGB cycle
	GOTO SM_END			; End
SM_L0S4	BTFSS f_SW_SL4			;Check if slow push 4
;	GOTO SM_END			; No, exit - DISABLES SAF Writes for DEBUG
	GOTO SM_L0S5			; No
	MOVLF c_MENU_LVL5,b_MENU_LVL	; Yes, set Menu level 5 (DMX address)
	BCF f_RGB_RUN			; Stop standard RGB cycle
	MOVFF b_DMX_UNIT,b_DMX_TMPC	; Load units into temp display
	MOVLF c_DMXA_DLY,b_DMX_DLYCNT	; Reset delay counter
	GOTO SM_END			; End
SM_L0S5	BTFSS f_SW_SL5			;Check if slow push 5
	GOTO SM_END			; No, exit
	BSF f_E2_WRITE			; Yes, set EEPROM write (save params)
	GOTO SM_END			; End
SM_L0F	BTFSC f_DMX_INH			;Check if DMX inhibit is active
	GOTO SM_END			; Yes, leave (DMX SLAVE INHIBITS FAST PUSH, only menu selection is enabled)
	BTFSC f_RGB_RUN			; No, level 0, fast push, check RGB status
	GOTO SM_L0F1			;  Running
	BSF f_RGB_ON			;  Not running, start with soft-on
	MOVFF b_RGB_STEPB,b_RGB_STEP	;  Restore RGB step counter from backup
	GOTO SM_END			;  End
SM_L0F1	BCF f_RGB_RUN			;Running, stops with soft-off
	BSF f_RGB_OFF			;
	MOVFF b_RGB_STEP,b_RGB_STEPB	;Backup RGB step counter
	GOTO SM_END			;End
SM_L1	BTFSS f_SW_FAST			;Level 1, check if fast push
	GOTO SM_END			; No, exit (level 1 use only fast push)
	BTFSC f_RGB_SPD1		; Yes, check RGB speed
	GOTO SM_L11			; 1
	BTFSC f_RGB_SPD2		;
	GOTO SM_L12			; 2
	BTFSC f_RGB_SPD3		;
	GOTO SM_L13			; 3
	BTFSC f_RGB_SPD4		;
	GOTO SM_L14			; 4
	ANDLF c_SPD_MASK,w_RGB_CTRL	; Speed is 5, set to 1
	BSF f_RGB_SPD1			;
	MOVLF c_RGB_FASTER,b_RGB_SPD	;Load faster delay to RGB cycle timer
	GOTO SM_END			;End
SM_L11	ANDLF c_SPD_MASK,w_RGB_CTRL	;Speed is 1, set to 2
	BSF f_RGB_SPD2			;
	MOVLF c_RGB_FAST,b_RGB_SPD	;Load fast delay to RGB cycle timer
	GOTO SM_END			;End
SM_L12	ANDLF c_SPD_MASK,w_RGB_CTRL	;Speed is 2, set to 3
	BSF f_RGB_SPD3			;
	MOVLF c_RGB_MEDIUM,b_RGB_SPD	;Load medium delay to RGB cycle timer
	GOTO SM_END			;End
SM_L13	ANDLF c_SPD_MASK,w_RGB_CTRL	;Speed is 3, set to 4
	BSF f_RGB_SPD4			;
	MOVLF c_RGB_SLOW,b_RGB_SPD	;Load slow delay to RGB cycle timer
	GOTO SM_END			;End
SM_L14	ANDLF c_SPD_MASK,w_RGB_CTRL	;Speed is 4, set to 5
	BSF f_RGB_SPD5			;
	MOVLF c_RGB_SLOWER,b_RGB_SPD	;Load slower delay to RGB cycle timer
	GOTO SM_END			;End
SM_L2	BTFSS f_SW_FAST			;Level 2, check if fast push
	GOTO SM_END			; No, exit (level 2 use only fast push)
	CLRF b_RGB_STEP			; Yes, force first step
	CLRF b_RGB_STEPB		; Same for backup
	BSF f_RGB_DONE			; Set step finished (force change to next step)
	CLRF b_RGB_CORR			; Clear color correction
	BTFSC f_RGB_MODE0		; Check RGB mode
	GOTO SM_L20			; 0
	BTFSC f_RGB_MODE1		; Check RGB mode
	GOTO SM_L21			; 1
	BTFSC f_RGB_MODE2		; Check RGB mode
	GOTO SM_L22			; 2
	BTFSC f_RGB_MODE3		; Check RGB mode
	GOTO SM_L23			; 3
	ANDLF c_MODE_MASK,b_MENU_FLAGS	; None of the above (4), clear modes
	BSF f_RGB_MODE0			; Set mode 0
	MOVLF 0xFF,b_PWM_RED		; Set Red 100% as first start color
	CLRF b_PWM_GRN			;
	CLRF b_PWM_BLU			;
	GOTO SM_END			; Exit
SM_L20	ANDLF c_MODE_MASK,b_MENU_FLAGS	;Clear modes
	BSF f_RGB_MODE1			;Set mode 1
	MOVLF 0xFF,b_PWM_RED		;Set Red 100% as first start color
	CLRF b_PWM_GRN			;
	CLRF b_PWM_BLU			;
	GOTO SM_END			;Exit
SM_L21	ANDLF c_MODE_MASK,b_MENU_FLAGS	;Clear modes
	BSF f_RGB_MODE2			;Set mode 2
	MOVLF 0xFF,b_PWM_GRN		;Set Green 100% as first start color
	CLRF b_PWM_RED			;
	CLRF b_PWM_BLU			;
	GOTO SM_END			;Exit
SM_L22	ANDLF c_MODE_MASK,b_MENU_FLAGS	;Clear modes
	BSF f_RGB_MODE3			;Set mode 3
	MOVLF 0xFF,b_PWM_BLU		;Set Blue 100% as first start color
	CLRF b_PWM_RED			;
	CLRF b_PWM_GRN			;
	GOTO SM_END			;Exit
SM_L23	ANDLF c_MODE_MASK,b_MENU_FLAGS	;Clear modes
	BSF f_RGB_MODE4			;Set mode 4
	MOVLF 0xFF,b_PWM_RED		;Set Red 100% as first start color
	CLRF b_PWM_GRN			;
	CLRF b_PWM_BLU			;
	GOTO SM_END			;Exit
SM_L3	BTFSS f_SW_FAST			;Level 3, check if fast push
	GOTO SM_END			; No, exit (level 3 use only fast push)
	MOVFF b_RGB_COLOR,R1L		; Yes, check color
	BZ SM_L300			;  0
	DECF R1L,f			;
	BZ SM_L301			;  1
	DECF R1L,f			;
	BZ SM_L302			;  2
	DECF R1L,f			;
	BZ SM_L303			;  3
	DECF R1L,f			;
	BZ SM_L304			;  4
	DECF R1L,f			;
	BZ SM_L305			;  5
	DECF R1L,f			;
	BZ SM_L306			;  6
	DECF R1L,f			;
	BZ SM_L307			;  7
	DECF R1L,f			;
	BZ SM_L308			;  8
	DECF R1L,f			;
	BZ SM_L309			;  9
	DECF R1L,f			;
	BZ SM_L310			;  10
	DECF R1L,f			;
	BZ SM_L311			;  11
	DECF R1L,f			;
	BZ SM_L312			;  12
	DECF R1L,f			;
	BZ SM_L313			;  13
	DECF R1L,f			;
	BZ SM_L314			;  14
	DECF R1L,f			;
	BZ SM_L315			;  15
	DECF R1L,f			;
	BZ SM_L316			;  16
	DECF R1L,f			;
	BZ SM_L317			;  17
	DECF R1L,f			;
	BZ SM_L318			;  18
	DECF R1L,f			;
	BZ SM_L319			;  19
	DECF R1L,f			;
	BZ SM_L320			;  20
	DECF R1L,f			;
	BZ SM_L321			;  21
	DECF R1L,f			;
	BZ SM_L322			;  22
	DECF R1L,f			;
	BZ SM_L323			;  23
	CLRF b_RGB_COLOR		;  24, set 0
	MOVLF c_RGB_REDR,b_PWM_RBK	;Color 0 is red
	MOVLF c_RGB_REDG,b_PWM_GBK	;
	MOVLF c_RGB_REDB,b_PWM_BBK	;
	GOTO SM_END
SM_L300	INCF b_RGB_COLOR,f		;Set color 1
	MOVLF c_RGB_RG25R,b_PWM_RBK	;Color 1
	MOVLF c_RGB_RG25G,b_PWM_GBK	;
	MOVLF c_RGB_RG25B,b_PWM_BBK	;
	GOTO SM_END
SM_L301	INCF b_RGB_COLOR,f		;Set color 2
	MOVLF c_RGB_RG50R,b_PWM_RBK	;Color 2
	MOVLF c_RGB_RG50G,b_PWM_GBK	;
	MOVLF c_RGB_RG50B,b_PWM_BBK	;
	GOTO SM_END
SM_L302	INCF b_RGB_COLOR,f		;Set color 3
	MOVLF c_RGB_RG75R,b_PWM_RBK	;Color 3
	MOVLF c_RGB_RG75G,b_PWM_GBK	;
	MOVLF c_RGB_RG75B,b_PWM_BBK	;
	GOTO SM_END
SM_L303	INCF b_RGB_COLOR,f		;Set color 4
	MOVLF c_RGB_YLWR,b_PWM_RBK	;Color 4 is yellow
	MOVLF c_RGB_YLWG,b_PWM_GBK	;
	MOVLF c_RGB_YLWB,b_PWM_BBK	;
	GOTO SM_END
SM_L304	INCF b_RGB_COLOR,f		;Set color 5
	MOVLF c_RGB_GR75R,b_PWM_RBK	;Color 5
	MOVLF c_RGB_GR75G,b_PWM_GBK	;
	MOVLF c_RGB_GR75B,b_PWM_BBK	;
	GOTO SM_END
SM_L305	INCF b_RGB_COLOR,f		;Set color 6
	MOVLF c_RGB_GR50R,b_PWM_RBK	;Color 6
	MOVLF c_RGB_GR50G,b_PWM_GBK	;
	MOVLF c_RGB_GR50B,b_PWM_BBK	;
	GOTO SM_END
SM_L306	INCF b_RGB_COLOR,f		;Set color 7
	MOVLF c_RGB_GR25R,b_PWM_RBK	;Color 7
	MOVLF c_RGB_GR25G,b_PWM_GBK	;
	MOVLF c_RGB_GR25B,b_PWM_BBK	;
	GOTO SM_END
SM_L307	INCF b_RGB_COLOR,f		;Set color 8
	MOVLF c_RGB_GRNR,b_PWM_RBK	;Color 8 is green
	MOVLF c_RGB_GRNG,b_PWM_GBK	;
	MOVLF c_RGB_GRNB,b_PWM_BBK	;
	GOTO SM_END
SM_L308	INCF b_RGB_COLOR,f		;Set color 9
	MOVLF c_RGB_GB25R,b_PWM_RBK	;Color 9
	MOVLF c_RGB_GB25G,b_PWM_GBK	;
	MOVLF c_RGB_GB25B,b_PWM_BBK	;
	GOTO SM_END
SM_L309	INCF b_RGB_COLOR,f		;Set color 10
	MOVLF c_RGB_GB50R,b_PWM_RBK	;Color 10
	MOVLF c_RGB_GB50G,b_PWM_GBK	;
	MOVLF c_RGB_GB50B,b_PWM_BBK	;
	GOTO SM_END
SM_L310	INCF b_RGB_COLOR,f		;Set color 11
	MOVLF c_RGB_GB75R,b_PWM_RBK	;Color 11
	MOVLF c_RGB_GB75G,b_PWM_GBK	;
	MOVLF c_RGB_GB75B,b_PWM_BBK	;
	GOTO SM_END
SM_L311	INCF b_RGB_COLOR,f		;Set color 12
	MOVLF c_RGB_AZRR,b_PWM_RBK	;Color 12 is azure
	MOVLF c_RGB_AZRG,b_PWM_GBK	;
	MOVLF c_RGB_AZRB,b_PWM_BBK	;
	GOTO SM_END
SM_L312	INCF b_RGB_COLOR,f		;Set color 13
	MOVLF c_RGB_BG75R,b_PWM_RBK	;Color 13
	MOVLF c_RGB_BG75G,b_PWM_GBK	;
	MOVLF c_RGB_BG75B,b_PWM_BBK	;
	GOTO SM_END
SM_L313	INCF b_RGB_COLOR,f		;Set color 14
	MOVLF c_RGB_BG50R,b_PWM_RBK	;Color 14
	MOVLF c_RGB_BG50G,b_PWM_GBK	;
	MOVLF c_RGB_BG50B,b_PWM_BBK	;
	GOTO SM_END
SM_L314	INCF b_RGB_COLOR,f		;Set color 15
	MOVLF c_RGB_BG25R,b_PWM_RBK	;Color 15
	MOVLF c_RGB_BG25G,b_PWM_GBK	;
	MOVLF c_RGB_BG25B,b_PWM_BBK	;
	GOTO SM_END
SM_L315	INCF b_RGB_COLOR,f		;Set color 16
	MOVLF c_RGB_BLUR,b_PWM_RBK	;Color 16 is blue
	MOVLF c_RGB_BLUG,b_PWM_GBK	;
	MOVLF c_RGB_BLUB,b_PWM_BBK	;
	GOTO SM_END
SM_L316	INCF b_RGB_COLOR,f		;Set color 17
	MOVLF c_RGB_BR25R,b_PWM_RBK	;Color 17
	MOVLF c_RGB_BR25G,b_PWM_GBK	;
	MOVLF c_RGB_BR25B,b_PWM_BBK	;
	GOTO SM_END
SM_L317	INCF b_RGB_COLOR,f		;Set color 18
	MOVLF c_RGB_BR50R,b_PWM_RBK	;Color 18
	MOVLF c_RGB_BR50G,b_PWM_GBK	;
	MOVLF c_RGB_BR50B,b_PWM_BBK	;
	GOTO SM_END
SM_L318	INCF b_RGB_COLOR,f		;Set color 19
	MOVLF c_RGB_BR75R,b_PWM_RBK	;Color 19
	MOVLF c_RGB_BR75G,b_PWM_GBK	;
	MOVLF c_RGB_BR75B,b_PWM_BBK	;
	GOTO SM_END
SM_L319	INCF b_RGB_COLOR,f		;Set color 20
	MOVLF c_RGB_VLTR,b_PWM_RBK	;Color 20 is violet
	MOVLF c_RGB_VLTG,b_PWM_GBK	;
	MOVLF c_RGB_VLTB,b_PWM_BBK	;
	GOTO SM_END
SM_L320	INCF b_RGB_COLOR,f		;Set color 21
	MOVLF c_RGB_RB75R,b_PWM_RBK	;Color 21
	MOVLF c_RGB_RB75G,b_PWM_GBK	;
	MOVLF c_RGB_RB75B,b_PWM_BBK	;
	GOTO SM_END
SM_L321	INCF b_RGB_COLOR,f		;Set color 22
	MOVLF c_RGB_RB50R,b_PWM_RBK	;Color 22
	MOVLF c_RGB_RB50G,b_PWM_GBK	;
	MOVLF c_RGB_RB50B,b_PWM_BBK	;
	GOTO SM_END
SM_L322	INCF b_RGB_COLOR,f		;Set color 23
	MOVLF c_RGB_RB25R,b_PWM_RBK	;Color 23
	MOVLF c_RGB_RB25G,b_PWM_GBK	;
	MOVLF c_RGB_RB25B,b_PWM_BBK	;
	GOTO SM_END
SM_L323	INCF b_RGB_COLOR,f		;Set color 24
	MOVLF c_RGB_WHTR,b_PWM_RBK	;Color 24 is white
	MOVLF c_RGB_WHTG,b_PWM_GBK	;
	MOVLF c_RGB_WHTB,b_PWM_BBK	;
	GOTO SM_END
SM_L4	BTFSS f_SW_FAST			;Level 4, check if fast push
	GOTO SM_END			; No, exit (level 4 use only fast push)
	BTFSS f_DMX_ON			; Yes, check if DMX is ON
	GOTO SM_L40			;  No
	BTFSC f_DMX_MASTER		;  Yes, check if master
	GOTO SM_L41 			;   Yes
	BCF f_DMX_ON			;   No, slave, set DMX to OFF
	BCF f_DMX_INH			;   Clear DMX inhibit flag
	BANKSEL PIR3			;Bank 14
	BCF PIE3,RC1IE			;Disable serial receive IRQ
	BANKSEL PORTA			;Bank 0
	GOTO SM_END			;   Exit
SM_L40	BSF f_DMX_ON			;DMX is OFF, set it ON
	BSF f_DMX_MASTER		;Set DMX as master
	BSF PCLATH,3			;Page 1
	CALL Dmx_StartT			;Start DMX transmit
	BCF PCLATH,3			;Page 0
	GOTO SM_END			;Exit
SM_L41	BCF f_DMX_MASTER		;DMX is master, set as slave
	GOTO SM_END			;Exit
SM_L5	BTFSS f_SW_FAST			;Level 5, check if fast push
	GOTO SM_L51			; No
	BSF PCLATH,3			; Yes, page 1
	CALL Dmx_Addr_BCD		; Convert DMX address into hundreds,
					;  tens and units, and reload temp registers
	BCF PCLATH,3			; Page 0
	MOVLF c_DMXA_DLY,b_DMX_DLYCNT	; Reset delay counter
	CLRF b_RGB_STEP			; Set step to 0
	MOVLW 0x09			; Check if address is already 9
	XORWF b_DMX_UNIT,w		;
	BZ SM_L50			;  Yes
	INCF b_DMX_UNIT,f		;  No, increment units
	MOVFF b_DMX_UNIT,b_DMX_TMPC	;  Load units into temp display
	BSF PCLATH,3			;  Page 1
	CALL Dmx_Addr_DBC		;  Convert new address into binary address
	BCF PCLATH,3			;  Page 0
	GOTO SM_END			;  Exit
SM_L50	CLRF b_DMX_UNIT			;Address is already 9, reset to 0
	CLRF b_DMX_TMPC			;Reset also temp display
	BSF PCLATH,3			;Page 1
	CALL Dmx_Addr_DBC		;Convert new address into binary address
	BCF PCLATH,3			;Page 0
	GOTO SM_END			;Exit
SM_L51	BTFSC f_SW_SL0			;Check if slow push 0
	GOTO SM_L52			; Yes
	BTFSC f_SW_SL1			; No, check if slow push 1
	GOTO SM_L52			;  Yes
	BTFSC f_SW_SL2			;  No, check if slow push 2
	GOTO SM_L52			;   Yes
	BTFSC f_SW_SL3			;   No, check if slow push 3
	GOTO SM_L52			;    Yes
	BTFSC f_SW_SL4			;    No, check if slow push 4
	GOTO SM_L52			;     Yes
	BTFSS f_SW_SL5			;     No, check if slow push 5
	GOTO SM_END			;      None of the above, exit
SM_L52	MOVLF c_MENU_LVL6,b_MENU_LVL	; Slow push, set menu level 6
	MOVLF c_DMXA_DLY,b_DMX_DLYCNT	; Reset delay counter
	MOVFF b_DMX_TENT,b_DMX_TMPC	; Load tens into temp display
	GOTO SM_END			;Exit
SM_L6	BTFSS f_SW_FAST			;Level 6, check if fast push
	GOTO SM_L61			; No
	BSF PCLATH,3			; Yes, page 1
	CALL Dmx_Addr_BCD		; Convert DMX address into hundreds,
					;  tens and units, and reload temp registers
	BCF PCLATH,3			; Page 0
	MOVLF c_DMXA_DLY,b_DMX_DLYCNT	; Reset delay counter
	CLRF b_RGB_STEP			; Set step to 0
	MOVLW 0x09			; Check if address is already 9
	XORWF b_DMX_TENT,w		;
	BZ SM_L60			;  Yes
	INCF b_DMX_TENT,f		;  No, increment tens
	MOVFF b_DMX_TENT,b_DMX_TMPC	;  Load tens into temp display
	BSF PCLATH,3			;  Page 1
	CALL Dmx_Addr_DBC		;  Convert new address into binary address
	BCF PCLATH,3			;  Page 0
	GOTO SM_END			;  Exit
SM_L60	CLRF b_DMX_TENT			;Address is already 9, reset to 0
	BSF PCLATH,3			;Page 1
	CALL Dmx_Addr_DBC		;Convert new address into binary address
	BCF PCLATH,3			;Page 0
	GOTO SM_END			;Exit
SM_L61	BTFSC f_SW_SL0			;Check if slow push 0
	GOTO SM_L62			; Yes
	BTFSC f_SW_SL1			; No, check if slow push 1
	GOTO SM_L62			;  Yes
	BTFSC f_SW_SL2			;  No, check if slow push 2
	GOTO SM_L62			;   Yes
	BTFSC f_SW_SL3			;   No, check if slow push 3
	GOTO SM_L62			;    Yes
	BTFSC f_SW_SL4			;    No, check if slow push 4
	GOTO SM_L62			;     Yes
	BTFSS f_SW_SL5			;     No, check if slow push 5
	GOTO SM_END			;      None of the above, exit
SM_L62	MOVLF c_MENU_LVL7,b_MENU_LVL	; Yes, set menu level 7
	MOVLF c_DMXA_DLY,b_DMX_DLYCNT	; Reset delay counter
	MOVFF b_DMX_HNDT,b_DMX_TMPC	; Load hundreds into temp display
	GOTO SM_END			;Exit
SM_L7	BTFSS f_SW_FAST			;Level 6, check if fast push
	GOTO SM_L71			; No
	BSF PCLATH,3			; Yes, page 1
	CALL Dmx_Addr_BCD		; Convert DMX address into hundreds,
					;  tens and units, and reload temp registers
	BCF PCLATH,3			; Page 0
	MOVLF c_DMXA_DLY,b_DMX_DLYCNT	; Reset delay counter
	CLRF b_RGB_STEP			; Set step to 0
	MOVLW 0x05			; Check if address is already 5
	XORWF b_DMX_HNDT,w		;
	BZ SM_L70			;  Yes
	INCF b_DMX_HNDT,f		;  No, increment hundreds
	MOVFF b_DMX_HNDT,b_DMX_TMPC	;  Load hundreds into temp display
	BSF PCLATH,3			;  Page 1
	CALL Dmx_Addr_DBC		;  Convert new address into binary address
	BCF PCLATH,3			;  Page 0
	GOTO SM_END			;  Exit
SM_L70	CLRF b_DMX_HNDT			;Address is already 5, reset to 0
	BSF PCLATH,3			;Page 1
	CALL Dmx_Addr_DBC		;Convert new address into binary address
	BCF PCLATH,3			;Page 0
	GOTO SM_END			;Exit
SM_L71	BTFSC f_SW_SL0			;Check if slow push 0
	GOTO SM_L72			; Yes
	BTFSC f_SW_SL1			; No, check if slow push 1
	GOTO SM_L72			;  Yes
	BTFSC f_SW_SL2			;  No, check if slow push 2
	GOTO SM_L72			;   Yes
	BTFSC f_SW_SL3			;   No, check if slow push 3
	GOTO SM_L72			;    Yes
	BTFSC f_SW_SL4			;    No, check if slow push 4
	GOTO SM_L72			;     Yes
	BTFSS f_SW_SL5			;     No, check if slow push 5
	GOTO SM_END			;      None of the above, exit
SM_L72	MOVLF c_MENU_LVL5,b_MENU_LVL	; Yes, set menu level 5
	MOVLF c_DMXA_DLY,b_DMX_DLYCNT	; Reset delay counter
	MOVFF b_DMX_UNIT,b_DMX_TMPC	; Load units into temp display
	GOTO SM_END			;Exit
SM_END	BCF f_SW_TOUT			;Clear push timeout flag
	BCF f_SW_CHG			;Clear push status change
	RETURN

;_______________________________________________________RGB Manager sub
;Uses input from the pushbutton+menu to set the PWM control sub
; in the right way
;Modified registers:
;	system
;	b_MENU_FLAGS
;	b_MENU_LVL
;	b_PWM_RED
;	b_PWM_RBK
;	b_PWM_GRN
;	b_PWM_GBK
;	b_PWM_BLU
;	b_PWM_BBK
;	b_RGB_STEP
;	b_RGB_BLINK
;	b_DMX_HNDT
;	b_DMX_TENT
;	b_DMX_UNIT
;Call method
;	CALL RGB_Manager
RGB_Manager:
	BANKSEL b_RGB_BLINK		;Bank 0
	DECFSZ b_RGB_BLINK,f		;Decrement Blink timer
	GOTO MG_NBLK			; If not zero, skip
	MOVLF 0x01,b_RGB_BLINK		; Zero, reload
	BTFSC f_MENU_LVL0		; Check which menu level we are in
	GOTO MG_LV0			;  Level 0
	BTFSC f_MENU_LVL1		;
	GOTO MG_LV1			;  Level 1
	BTFSC f_MENU_LVL2		;
	GOTO MG_LV2			;  Level 2
	BTFSC f_MENU_LVL3		;
	GOTO MG_MD5			;  Level 3
	BTFSC f_MENU_LVL4		;
	GOTO MG_LV4			;  Level 4
	BTFSC f_MENU_LVL5		;
	GOTO MG_LV5			;  Level 5
	BTFSC f_MENU_LVL6		;
	GOTO MG_LV5			;  Level 6
	BTFSC f_MENU_LVL7		;
	GOTO MG_LV5			;  Level 7
	MOVLF c_MENU_LVL0,b_MENU_LVL	;  None, this is an error, set level 0
MG_LV0	BTFSC f_SW_SL0			;Level 0, check if/which slow press present
	GOTO MG_SP0			; Slow press 0
	BTFSC f_SW_SL1			;
	GOTO MG_SP1			; Slow press 1
	BTFSC f_SW_SL2			;
	GOTO MG_SP2			; Slow press 2
	BTFSC f_SW_SL3			;
	GOTO MG_SP3			; Slow press 3
	BTFSC f_SW_SL4			;
	GOTO MG_SP4			; Slow press 4
	BTFSC f_SW_SL5			;
	GOTO MG_SP5			; Slow press 5
	GOTO MG_NBLK			; None, keep checking
MG_SP0	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
MG_SP00	BCF f_RGB_BLNK			; No, clear
	MOVLF c_RGB_LREDR,R1H		; Load low RED setpoint
	CLRF R1L			;
	CLRF R2H			;
	GOTO MG_SPZ			; Set Blink timer
MG_SP1	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
MG_SP10	BCF f_RGB_BLNK			; No, clear
	CLRF R1H			; Load low GREEN setpoint
	MOVLF c_RGB_LGRNG,R1L		;
	CLRF R2H			;
	GOTO MG_SPZ			; Set Blink timer
MG_SP2	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
MG_SP20	BCF f_RGB_BLNK			; No, clear
	CLRF R1H			; Load low BLUE setpoint
	CLRF R1L			;
	MOVLF c_RGB_LBLUB,R2H		;
	GOTO MG_SPZ			; Set Blink timer
MG_SP3	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
MG_SP30	BCF f_RGB_BLNK			; No, clear
	MOVLF c_RGB_LYLWR,R1H		;Load low YELLOW setpoint
	MOVLF c_RGB_LYLWG,R1L		;
	CLRF R2H			;
	GOTO MG_SPZ			;Set Blink timer
MG_SP4	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
MG_SP40	BCF f_RGB_BLNK			; No, clear
	MOVLF c_RGB_LVLTR,R1H		;Load low VIOLET setpoint
	CLRF R1L			;
	MOVLF c_RGB_LVLTB,R2H		;
	GOTO MG_SPZ			;Set Blink timer
MG_SP5	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
MG_SP50	BCF f_RGB_BLNK			;Clear RGB blink
	MOVLF c_RGB_LWHTR,R1H		;Load low WHITE setpoint
	MOVLF c_RGB_LWHTG,R1L		;
	MOVLF c_RGB_LWHTB,R2H		;
MG_SPZ	MOVLF c_TMR_BLK1,b_RGB_BLINK	; Reload blink timer for 100ms
	BCF f_RGB_ON			; Disable soft-on
	BCF f_RGB_OFF			; Disable soft-off
	GOTO RC_END			; End
MG_SPA	BSF f_RGB_BLNK			;Set RGB blink
	CLRF R1H			;Set all PWM to 0
	CLRF R1L			;
	CLRF R2H			;
	MOVLF c_TMR_BLK3,b_RGB_BLINK	;Reload blink timer for 300ms
	BCF f_RGB_ON			;Disable soft-on
	BCF f_RGB_OFF			;Disable soft-off
	GOTO RC_END			;End
MG_LV1	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
	BTFSC f_RGB_SPD1		;  No, check if RGB speed 1
	GOTO MG_SP00			;   Yes, Red
	BTFSC f_RGB_SPD2		;   No, check if RGB speed 2
	GOTO MG_SP10			;    Yes, Green
	BTFSC f_RGB_SPD3		;    No, check if RGB speed 3
	GOTO MG_SP20			;     Yes, Blue
	BTFSC f_RGB_SPD4		;     No, check if RGB speed 4
	GOTO MG_SP30			;      Yes, Yellow
	GOTO MG_SP40			;      No, must be 5, Violet
MG_LV2	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
	BTFSC f_RGB_MODE0		;  No, check if RGB mode 0
	GOTO MG_SP30			;   Yes, Yellow
	BTFSC f_RGB_MODE1		;   No, check if RGB mode 1
	GOTO MG_SP00			;    Yes, Red
	BTFSC f_RGB_MODE2		;    No, check if RGB mode 2
	GOTO MG_SP10			;     Yes, Green
	BTFSC f_RGB_MODE3		;     No, check if RGB mode 3
	GOTO MG_SP20			;      Yes, Blue
	BTFSC f_RGB_MODE4		;      No, check if RGB mode 4
	GOTO MG_SP50			;       Yes, White
	GOTO MG_SP40			;       No, must be RGB mode 5, Violet
MG_LV4	BTFSS f_RGB_BLNK		;Check if RGB blink is clear
	GOTO MG_SPA			; Yes
	BTFSS f_DMX_ON			; No, check if DMX is ON
	GOTO MG_SP00			;  No, Red
	BTFSC f_DMX_MASTER		;  Yes, check if master
	GOTO MG_SP10			;   Yes, Green
	GOTO MG_SP20			;   No, Blue
MG_LV5	BTFSC f_DMX_DLYAD		;Check if delay end is set
	GOTO MG_LV54			; Yes, do nothing
	CLRF b_SW_TMR			; No, reset timeout timer
	MOVF b_RGB_STEP,w		;Check if RGB Step is 0
	BZ MG_LV53			; Yes
	CLRF b_RGB_STEP			; No, set to 0
	MOVLF c_TMR_BLK2,b_RGB_BLINK	;Reload blink timer for 200ms
	MOVF b_DMX_TMPC,w		;Check if temp display = 0
	BZ MG_LV51			; Yes
	MOVLW c_RGB_LREDR		; No, load low Setpoint
	BTFSC f_MENU_LVL5		; Check which color to show
	MOVWF R1H			;  Save on RED PWM
	BTFSC f_MENU_LVL6
	MOVWF R1L			;  Save on GREEN PWM
	BTFSC f_MENU_LVL7
	MOVWF R2H			;  Save on BLUE PWM
	DECF b_DMX_TMPC,f		;Decrement temp display by 1
	GOTO RC_END			;End
MG_LV51	BTFSC f_MENU_LVL5		;Check which address to reload
	MOVF b_DMX_UNIT,w		;Reload units
	BTFSC f_MENU_LVL6
	MOVF b_DMX_TENT,w		;Reload tens
	BTFSC f_MENU_LVL7
	MOVF b_DMX_HNDT,w		;Reload hundreds
	MOVWF b_DMX_TMPC		;Copy to temp display counter
	MOVLF c_TMR_BLK8,b_RGB_BLINK	;Reload blink timer for 800ms
	DECFSZ b_DMX_DLYCNT		;Decrement delay counter
	GOTO MG_LV52			; Not 0
	BSF f_DMX_DLYAD			; 0, set delay end
MG_LV52	CLRF b_RGB_STEP			;Set RGB step to 0
	MOVLF c_RGB_LWHTR,R1H		;Load low WHITE setpoint
	MOVLF c_RGB_LWHTG,R1L		;
	MOVLF c_RGB_LWHTB,R2H		;
	GOTO RC_END			;End
MG_LV53	MOVLF c_TMR_BLK2,b_RGB_BLINK	;Reload blink timer for 200ms
MG_LV54	MOVLF 0x01,b_RGB_STEP		;Set RGB step to 1
	CLRF R1H			;Set all PWM to 0
	CLRF R1L			;
	CLRF R2H			;
	GOTO RC_END			;End
MG_NBLK	BTFSC f_DMX_INH			;Check if inhibit is active
	RETURN				; Yes, exit (DMX SLAVE, function is disabled)
	BTFSS f_RGB_DONE		;No, check if RGB cycle ended
	RETURN				; No, end
	BTFSS f_RGB_RUN			; Yes, check if RGB is active
	RETURN				;  No, end
	BTFSC f_RGB_ERR			;  Yes, check if error
	GOTO MG_ERR			;   Yes
	BTFSC f_RGB_MODE0		;   No, check RGB Mode
	GOTO MG_MD0			;    Mode 0
	BTFSC f_RGB_MODE1		;
	GOTO MG_MD1			;    Mode 1
	BTFSC f_RGB_MODE2		;
	GOTO MG_MD2			;    Mode 2
	BTFSC f_RGB_MODE3		;
	GOTO MG_MD3			;    Mode 3
	BTFSC f_RGB_MODE4		;
	GOTO MG_MD4			;    Mode 4
	BTFSC f_RGB_MODE5		;
	GOTO MG_END			;    Mode 5
	GOTO MG_ERR			;    None of the above. ERROR, set mode 0
MG_MD0	MOVLF c_TMR_CORR,b_RGB_CNT	;RGB Mode 0, set rgb counter for color correction
	MOVF b_RGB_STEP,w		;Check step
	BNZ MG_MD01			; Next
	MOVLF 0x01,b_RGB_STEP		; Step 0, set next step 1
	MOVLF .16,b_RGB_MODE		; Set mode (RG+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD01	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD02			; Next
	MOVLF 0x02,b_RGB_STEP		; Step 1, set next step 2
	MOVLF .9,b_RGB_MODE		; Set mode (R-G)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD02	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD03			; Next
	MOVLF 0x03,b_RGB_STEP		; Step 2, set next step 3
	MOVLF .27,b_RGB_MODE		; Set mode (GB+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD03	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD04			; Next
	MOVLF 0x04,b_RGB_STEP		; Step 3, set next step 4
	MOVLF .22,b_RGB_MODE		; Set mode (G-B)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD04	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD05			; Next
	MOVLF 0x05,b_RGB_STEP		; Step 4, set next step 5
	MOVLF .3,b_RGB_MODE		; Set mode (R+B)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD05	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD06			; Next
	MOVLF 0x06,b_RGB_STEP		; Step 5, set next step 6
	MOVLF .29,b_RGB_MODE		; Set mode (RB-)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD06	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_EN2			; Step not defined, reset to 0
	MOVLF 0x01,b_RGB_STEP		; Step 6, set next step 1
	MOVLF .16,b_RGB_MODE		; Set mode (RG+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD1	MOVLF c_TMR_CORR,b_RGB_CNT	;RGB Mode 1, set rgb counter for color correction
	MOVF b_RGB_STEP,w		;Check step
	BNZ MG_MD11			; Next
	MOVLF 0x01,b_RGB_STEP		; Step 0, set next step 1
	MOVLF .16,b_RGB_MODE		; Set mode (RG+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD11	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD12			; Next
	MOVLF 0x02,b_RGB_STEP		; Step 1, set next step 2
	MOVLF .21,b_RGB_MODE		; Set mode (RG-)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD12	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD13			; Next
	MOVLF 0x03,b_RGB_STEP		; Step 2, set next step 3
	MOVLF .26,b_RGB_MODE		; Set mode (RB+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD13	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD14			; Next
	MOVLF 0x04,b_RGB_STEP		; Step 3, set next step 4
	MOVLF .29,b_RGB_MODE		; Set mode (RB-)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD14	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_EN2			; Step not defined, reset to 0
	MOVLF 0x01,b_RGB_STEP		; Step 4, set next step 1
	MOVLF .16,b_RGB_MODE		; Set mode (RG+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD2	MOVLF c_TMR_CORR,b_RGB_CNT	;RGB Mode 2, set rgb counter for color correction
	MOVF b_RGB_STEP,w		;Check step
	BNZ MG_MD21			; Next
	MOVLF 0x01,b_RGB_STEP		; Step 0, set next step 1
	MOVLF .27,b_RGB_MODE		; Set mode (GB+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD21	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD22			; Next
	MOVLF 0x02,b_RGB_STEP		; Step 1, set next step 2
	MOVLF .30,b_RGB_MODE		; Set mode (GB-)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD22	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD23			; Next
	MOVLF 0x03,b_RGB_STEP		; Step 2, set next step 3
	MOVLF .2,b_RGB_MODE		; Set mode (R+G)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD23	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD24			; Next
	MOVLF 0x04,b_RGB_STEP		; Step 3, set next step 4
	MOVLF .9,b_RGB_MODE		; Set mode (R-G)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD24	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD25			; Step not defined, reset to 0
	MOVLF 0x01,b_RGB_STEP		; Step 4, set next step 1
	MOVLF .27,b_RGB_MODE		; Set mode (GB+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD25	CLRF b_PWM_RED			; Clear PWM setpoint
	CLRF b_PWM_GRN			;
	CLRF b_PWM_BLU			;
	CLRF b_RGB_STEP			; Set first step
	MOVLF .15,b_RGB_MODE		; Reset to first mode (G+)
	GOTO MG_END			; End
MG_MD3	MOVLF c_TMR_CORR,b_RGB_CNT	;RGB Mode 3, set rgb counter for color correction
	MOVF b_RGB_STEP,w		;Check step
	BNZ MG_MD31			; Next
	MOVLF 0x01,b_RGB_STEP		; Step 0, set next step 1
	MOVLF .3,b_RGB_MODE		; Set mode (R+B)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD31	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD32			; Next
	MOVLF 0x02,b_RGB_STEP		; Step 1, set next step 2
	MOVLF .10,b_RGB_MODE		; Set mode (R-B)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD32	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD33			; Next
	MOVLF 0x03,b_RGB_STEP		; Step 2, set next step 3
	MOVLF .17,b_RGB_MODE		; Set mode (G+B)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD33	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD34			; Next
	MOVLF 0x04,b_RGB_STEP		; Step 3, set next step 4
	MOVLF .22,b_RGB_MODE		; Set mode (G-B)
	MOVLF c_2C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD34	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD35			; Step not defined, reset to 0
	MOVLF 0x01,b_RGB_STEP		; Step 4, set next step 1
	MOVLF .3,b_RGB_MODE		; Set mode (R+B)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD35	CLRF b_PWM_RED			; Clear PWM setpoint
	CLRF b_PWM_GRN			;
	CLRF b_PWM_BLU			;
	CLRF b_RGB_STEP			; Set first step
	MOVLF .25,b_RGB_MODE		; Reset to first mode (B+)
	GOTO MG_END			; End
MG_MD4	MOVLF c_TMR_CORR,b_RGB_CNT	;RGB Mode 4, set rgb counters 1 and 2 for color correction
	MOVWF b_RGB_CNT2		;
	MOVF b_RGB_STEP,w		;Check step
	BNZ MG_MD41			; Next
	MOVLF 0x01,b_RGB_STEP		; Step 0, set next step 1
	MOVLF .19,b_RGB_MODE		; Set mode (RG+B+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD41	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD42			; Next
	MOVLF 0x02,b_RGB_STEP		; Step 1, set next step 2
	MOVLF .14,b_RGB_MODE		; Set mode (R-GB-)
	MOVLF c_3C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD42	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD43			; Next
	MOVLF 0x03,b_RGB_STEP		; Step 2, set next step 3
	MOVLF .7,b_RGB_MODE		; Set mode (R+GB+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD43	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD44			; Next
	MOVLF 0x04,b_RGB_STEP		; Step 3, set next step 4
	MOVLF .12,b_RGB_MODE		; Set mode (R-G-B)
	MOVLF c_3C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD44	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD45			; Next
	MOVLF 0x05,b_RGB_STEP		; Step 4, set next step 5
	MOVLF .5,b_RGB_MODE		; Set mode (R+G+B)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD45	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_MD46			; Next
	MOVLF 0x06,b_RGB_STEP		; Step 5, set next step 6
	MOVLF .24,b_RGB_MODE		; Set mode (RG-B-)
	MOVLF c_3C_CORR,b_RGB_CORR	; Set color correction
	GOTO MG_END			; End
MG_MD46	DECFSZ b_RGB_STEP,f		;Check step
	GOTO MG_EN2			; Step not defined, reset to 0
	MOVLF 0x01,b_RGB_STEP		; Step 6, set next step 1
	MOVLF .19,b_RGB_MODE		; Set mode (RG+B+)
	CLRF b_RGB_CORR			; Reset color correction
	GOTO MG_END			; End
MG_MD5	BCF f_RGB_ON			;RGB Mode 5, disable soft-on
	BCF f_RGB_OFF			;Disable soft-off
	MOVLF .31,b_RGB_MODE		;Set mode (fixed colors)
	GOTO MG_END			;End
MG_ERR	BSF f_RGB_MODE0			;Error, set RGB mode 0 (standard RGB)
	BCF f_RGB_ERR			;Clear error flag
MG_EN2	CLRF b_PWM_RED			;Clear PWM setpoint
	CLRF b_PWM_GRN			;
	CLRF b_PWM_BLU			;
	CLRF b_RGB_STEP			;Set first step
	MOVLF .1,b_RGB_MODE		;Reset to first mode (R+)
MG_END	BCF f_RGB_DONE			;Clear RGB step done flag
	RETURN
