;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |      Include for RGB frame DMX control       |
; |______________________________________________|
; |   Include File                               |
; |______________________________________________|
; |   Type      : INC                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________Constants
c_DMX_BAUDH	EQU	0x00		;DMX baud rate MSB (250kbit/s @ 32MHz with high speed 8bit BRG)
c_DMX_BAUDL	EQU	0x07		; DMX baud rate LSB
c_DMX_RXSTA	EQU	B'10010000'    	;DMX config for receive serial register
					; (serial port ON, 8-bit, continuous receive,
					; addr. detection OFF)
c_DMX_TXSTA	EQU	B'01000101'    	;DMX config for transmit serial register
					; (async, 9-bit, TX enabled, high speed, 9th bit=1)
c_DMX_BCON	EQU	B'01000000'	;DMX config for baud serial register
					; (receiver idle, polarity high, 8bit baud
					; RX not monitored, auto baud disabled)

c_DMX_START	EQU	0x00		;DMX start packet (0=standard 8bit data packets)
c_DMX_ERR	EQU	0x03		;DMX error limit before resetting receive sequence
c_DMX_ADDRH	EQU	0x00		;DMX initial address
c_DMX_ADDRL	EQU	0x01		;
	
c_DMXM_CLR	EQU	0x1F		;DMX master mode clear AND mask
c_DMXS_CLR	EQU	0x07		;DMX slave mode clear AND mask

c_DMXA_DLY	EQU	0x03		;dmx address delay

;_______________________________________________________Variables
	BYTE b_DMX_HNDT			;DMX hundreds temp address
	BYTE b_DMX_TENT			;DMX tens temp address
	BYTE b_DMX_UNIT			;DMX units temp address
	WORD w_DMX_ADDR			;DMX full address
	WORD w_DMX_CNT			;DMX address counter
	BYTE b_DMX_CTRL			;DMX control flags
	BYTE b_DMX_RED			;DMX received RED setpoint
	BYTE b_DMX_GRN			;DMX received GREEN setpoint
	BYTE b_DMX_BLU			;DMX received BLUE setpoint
	BYTE b_DMX_ERR			;DMX receive error counter
	BYTE b_DMX_DLYCNT		;DMX address display delay counter
	BYTE b_DMX_TMPC			;DMX address display temp

;_______________________________________________________Flags
#define f_DMX_ON	b_DMX_CTRL,0	;DMX enabled
#define f_DMX_MASTER	b_DMX_CTRL,1	;DMX master mode
#define f_DMX_INH	b_DMX_CTRL,2	;DMX inhibit flag (inhibits other functions if DMX slave is on)
#define f_DMXS_RD	b_DMX_CTRL,3	;DMX slave RED packet wait
#define f_DMXS_GN	b_DMX_CTRL,4	;DMX slave GREEN packet wait
#define f_DMXM_BRK	b_DMX_CTRL,5	;DMX master Break send mode / Slave framing error (Break) wait
#define f_DMXM_MAB	b_DMX_CTRL,6	;DMX master Mark-After-Break send mode / Slave BLUE packet wait
#define f_DMXM_STR	b_DMX_CTRL,7	;DMX master Start packet send mode / Slave start packet wait

#define f_DMX_DLYAD	b_DMX_DLYCNT,7	;DMX address delay end - Use an unoccupied bit from the counter register