;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |      Include for RGB frame I/O control       |
; |______________________________________________|
; |   Include File                               |
; |______________________________________________|
; |   Type      : INC                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________Constants
c_TRISA		EQU	B'00100111'	;PortA I/O config (all pins except RA4 as inputs)
c_ANSA		EQU	B'00000000'	;PortA D/A config (all pins digital I/O)
c_WPUA		EQU	B'00101110'	;PortA Pull-Ups config (enabled on RA1+RA2+RA3+RA5)
c_ODCA		EQU	B'00000000'	;PortA Open-Drain config (all pins standard push-pull)
c_SLRA		EQU	B'00000000'	;PortA Slew Rate config (all pins on max slew rate)
c_INLVA		EQU	B'00000000'	;PortA Input level config (all pins on TTL levels)

c_TRISC		EQU	B'00101000'	;PortC I/O config (RC3+RC5 inputs, RC0+RC1+RC2+RC4 outputs)
c_ANSC		EQU	B'00000000'	;PortC D/A config (all pins digital I/O)
c_WPUC		EQU	B'00101000'	;PortC Pull-Ups config (enabled on RC3 and RC5)
c_ODCC		EQU	B'00000000'	;PortC Open-Drain config (all pins standard push-pull)
c_SLRC		EQU	B'00000000'	;PortC Slew Rate config (all pins on max slew rate)
c_INLVC		EQU	B'00000000'	;PortC Input level config (all pins on TTL levels)

c_RX1PPS	EQU	0x15		;Route pin RC5 to RX1  (RX1DTPPS)
c_TX1PPS	EQU	0x14		;Route pin RC4 to TX1  (TX1CKPPS)
c_RC0PPS	EQU	0x0D		;Route PWM5 to pin RC0
c_RC1PPS	EQU	0x0C	    	;Route PWM4 to pin RC1
c_RC2PPS	EQU	0x0B		;Route PWM3 to pin RC2
c_RC4PPS	EQU	0x0F		;Route TX1 to pin RC4

c_SW_BIT	EQU	4		;Pushbutton counter bit (2^n x 10ms buffer time) - 160ms
c_SW_SLOWL	EQU	0x0A		;Pushbutton slow counter Low value (n x 100ms buffer time) - 1000ms
c_SW_SLOWH	EQU	0x1E		;Pushbutton slow counter High value (n x 100ms buffer time) - 3000ms
c_SW_SLIN	EQU	0x01		;Pushbutton slow counter init value
c_SW_SLRP	EQU	0x0B		;Pushbutton slow counter repeat value

c_PUSH_PORT	EQU	PORTC		;Pushbutton port
c_PUSH_PIN	EQU	3		;Pushbutton pin (RC3)

c_DIR_PORT	EQU	PORTA		;TX port
c_DIR_PIN	EQU	4		;TX pin (RA4)

c_TX_PORT	EQU	PORTC		;TX port
c_TX_LATC	EQU	LATC		;TX latch
c_TX_TRIS	EQU	TRISC		;TX tris
c_TX_WPU	EQU	WPUC		;TX pull-up
c_TX_PPS	EQU	RC4PPS		;TX PPS
c_TX_PIN	EQU	4		;TX pin (RC4)


;_______________________________________________________Variables
	BYTE b_SW_FAST			;Pushbutton fast press counter
	BYTE b_SW_SLOW			;Pushbutton slow press counter
	BYTE b_SW_TMR			;Pushbuttonm timeout timer
	WORD w_SW_STAT			;Pushbutton status

;_______________________________________________________Flags
#define f_SW_STAT   w_SW_STAT,0		;Pushbutton status flag
#define f_SW_FAST   w_SW_STAT,1		;Pushbutton fast press (<1 sec)
#define f_SW_SL0    w_SW_STAT,2		;Pushbutton slow press 0 (1-3 sec) flag
#define f_SW_SL1    w_SW_STAT,3		;Pushbutton slow press 1 (3-5 sec) flag
#define f_SW_SL2    w_SW_STAT,4		;Pushbutton slow press 2 (5-7 sec) flag
#define f_SW_SL3    w_SW_STAT,5		;Pushbutton slow press 3 (7-9 sec) flag
#define f_SW_SL4    w_SW_STAT,6		;Pushbutton slow press 4 (9-11 sec) flag
#define f_SW_SL5    w_SW_STAT,7		;Pushbutton slow press 5 (>11 sec) flag

#define f_SW_TIM1   w_SW_STAT+1,0	;Pushbutton timer toggle (divide timing by 2)
#define f_SW_TIM2   w_SW_STAT+1,1	;Pushbutton timeout start/stop
#define f_SW_TOUT   w_SW_STAT+1,2	;Pushbutton timeout flag
#define f_SW_CHG    w_SW_STAT+1,3	;Pushbutton status change flag
