;*****************************************************************************
;*                                                                           *
;* RGB frame                                                                 *
;* Copyright (C) 2020 by Walter Lain <kcswalter@member.fsf.org>              *
;*                                                                           *
;* This program is free software; you can redistribute it and/or modify      *
;* it under the terms of the GNU General Public License as published by      *
;* the Free Software Foundation; either version 3 of the License, or         *
;* (at your option) any later version.                                       *
;*                                                                           *
;* This program is distributed in the hope that it will be useful,           *
;* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
;* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
;* GNU General Public License for more details.                              *
;*                                                                           *
;* You should have received a copy of the GNU General Public License         *
;* along with this program; if not, write to the Free Software               *
;* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
;*                                                                           *
;*****************************************************************************

; ________________________________________________
; |        Include for RGB frame system          |
; |______________________________________________|
; |   Include File                               |
; |______________________________________________|
; |   Type      : INC                            |
; |   Version   : 1                              |
; |   Release   : 0                              |
; |   Author    : Lain Walter                    |
; |   Date      : 10/01/21                       |
; |______________________________________________|

;_______________________________________________________Constants
c_PMD0		EQU	B'01000001'	;System clock network ON, Vref OFF, NVM writeable
					; Clock reference ON, IOC OFF
c_PMD1		EQU	B'10000000'	;NCO1 OFF, Timer0 ON, Timer1 ON, Timer2 ON
c_PMD2		EQU	B'01100111'	;DAC OFF, ADC OFF, Comp1 OFF, Comp2 OFF, ZCD OFF
c_PMD3		EQU	B'00100011'	;PWM6 OFF, PWM3-5 ON, CCP1 OFF, CCP2 OFF
c_PMD4		EQU	B'10010001'	;USART1 ON, USART2 OFF, MSSP1 OFF, CWG1 OFF
c_PMD5		EQU	B'00011110'	;CLC1-4 OFF
c_MAXRAM	EQU	2		;Amount of system RAM
					; 0=256 bytes -> PIC16F153x3
					; 1=512 bytes -> PIC16F153x4
					; 2=1024 bytes -> PIC16F153x5

c_MODE_MASK	EQU	B'11000000'	;Mask to clear all RGB modes
c_MENU_LVL0	EQU	0x01		;Menu level 0 mask
c_MENU_LVL1	EQU	0x02		;Menu level 1 mask
c_MENU_LVL2	EQU	0x04		;Menu level 2 mask
c_MENU_LVL3	EQU	0x08		;Menu level 3 mask
c_MENU_LVL4	EQU	0x10		;Menu level 4 mask
c_MENU_LVL5	EQU	0x20		;Menu level 5 mask
c_MENU_LVL6	EQU	0x40		;Menu level 6 mask
c_MENU_LVL7	EQU	0x80		;Menu level 7 mask

;_______________________________________________________Variables
	BYTE b_MENU_FLAGS		;Menu flags
	BYTE b_MENU_LVL			;Menu level flags
;_______________________________________________________Flags
#define f_RGB_MODE0	b_MENU_FLAGS,0	;RGB mode 0 (standard RGB cycle)
#define f_RGB_MODE1	b_MENU_FLAGS,1	;RGB mode 1 (RED always on cycle)
#define f_RGB_MODE2	b_MENU_FLAGS,2	;RGB mode 2 (GREEN always on cycle)
#define f_RGB_MODE3	b_MENU_FLAGS,3	;RGB mode 3 (BLUE always on cycle)
#define f_RGB_MODE4	b_MENU_FLAGS,4	;RGB mode 4 (RGBW cycle)
#define f_RGB_MODE5	b_MENU_FLAGS,5	;RGB mode 5 (Fixed color mode)
#define f_MENU_ZERO	b_MENU_FLAGS,6	;Zero flag (for DMX address display)
#define f_MENU_BLINK	b_MENU_FLAGS,7	;Blink enable flag

#define f_MENU_LVL0	b_MENU_LVL,0	;Menu level 0
#define f_MENU_LVL1	b_MENU_LVL,1	;Menu level 1
#define f_MENU_LVL2	b_MENU_LVL,2	;Menu level 2
#define f_MENU_LVL3	b_MENU_LVL,3	;Menu level 3
#define f_MENU_LVL4	b_MENU_LVL,4	;Menu level 4
#define f_MENU_LVL5	b_MENU_LVL,5	;Menu level 5
#define f_MENU_LVL6	b_MENU_LVL,6	;Menu level 6
#define f_MENU_LVL7	b_MENU_LVL,7	;Menu level 7
